@@ -147,6 +147,18 @@ VBoxDrvFindAdapter(IN PVOID HwDeviceExtension, IN PVOID HwContext, IN PWSTR Argu
      */
     VBoxSetupDisplaysHGSMI(&pExt->u.primary.commonInfo, phVRAM, ulApertureSize, cbVRAM, 0);
 
+    /* Check if the chip restricts horizontal resolution or not.
+     * Must be done after VBoxSetupDisplaysHGSMI, because it initializes the common structure.
+     */
+    VideoPortWritePortUshort((PUSHORT)VBE_DISPI_IOPORT_INDEX, VBE_DISPI_INDEX_ID);
+    VideoPortWritePortUshort((PUSHORT)VBE_DISPI_IOPORT_DATA, VBE_DISPI_ID_ANYX);
+    DispiId = VideoPortReadPortUshort((PUSHORT)VBE_DISPI_IOPORT_DATA);
+
+    if (DispiId == VBE_DISPI_ID_ANYX)
+        VBoxCommonFromDeviceExt(pExt)->fAnyX = TRUE;
+    else
+        VBoxCommonFromDeviceExt(pExt)->fAnyX = FALSE;
+
     if (pExt->u.primary.commonInfo.bHGSMI)
     {
         LOGREL(("using HGSMI"));
@@ -166,7 +178,6 @@ static BOOLEAN
 VBoxDrvInitialize(PVOID HwDeviceExtension)
 {
     PVBOXMP_DEVEXT pExt = (PVBOXMP_DEVEXT) HwDeviceExtension;
-    USHORT DispiId;
 
     PAGED_CODE();
     LOGF_ENTER();
@@ -174,16 +185,6 @@ VBoxDrvInitialize(PVOID HwDeviceExtension)
     /* Initialize the request pointer. */
     pExt->u.primary.pvReqFlush = NULL;
 
-    /* Check if the chip restricts horizontal resolution or not. */
-    VideoPortWritePortUshort((PUSHORT)VBE_DISPI_IOPORT_INDEX, VBE_DISPI_INDEX_ID);
-    VideoPortWritePortUshort((PUSHORT)VBE_DISPI_IOPORT_DATA, VBE_DISPI_ID_ANYX);
-    DispiId = VideoPortReadPortUshort((PUSHORT)VBE_DISPI_IOPORT_DATA);
-
-    if (DispiId == VBE_DISPI_ID_ANYX)
-        pExt->fAnyX = TRUE;
-    else
-        pExt->fAnyX = FALSE;
-
     VBoxMPCmnInitCustomVideoModes(pExt);
 
     LOGF_LEAVE();
@@ -308,7 +309,7 @@ VBoxDrvStartIO(PVOID HwDeviceExtension, PVIDEO_REQUEST_PACKET RequestPacket)
         {
             PVIDEO_MODE_INFORMATION pModes = (PVIDEO_MODE_INFORMATION) RequestPacket->OutputBuffer;
 
-            if (RequestPacket->OutputBufferLength < VBoxMPXpdmGetVideoModesCount()*sizeof(VIDEO_MODE_INFORMATION))
+            if (RequestPacket->OutputBufferLength < VBoxMPXpdmGetVideoModesCount(pExt)*sizeof(VIDEO_MODE_INFORMATION))
             {
                 pStatus->Status = ERROR_INSUFFICIENT_BUFFER;
                 break;
@@ -538,7 +539,7 @@ VBoxDrvStartIO(PVOID HwDeviceExtension, PVIDEO_REQUEST_PACKET RequestPacket)
         case IOCTL_VIDEO_VBOX_ISANYX:
         {
             STARTIO_OUT(uint32_t, pu32AnyX);
-            *pu32AnyX = pExt->fAnyX;
+            *pu32AnyX = VBoxCommonFromDeviceExt(pExt)->fAnyX;
             pStatus->Information = sizeof (uint32_t);
             bResult = TRUE;
             break;
