@@ -1303,8 +1303,11 @@ static DWORD vboxDispIfReninitModesWDDM(VBOXDISPIF_OP *pOp, const uint8_t *pScre
 
     D3DKMT_ESCAPE EscapeData = {0};
     EscapeData.hAdapter = pOp->Adapter.hAdapter;
+#ifdef VBOX_DISPIF_WITH_OPCONTEXT
+    /* win8.1 does not allow context-based escapes for display-only mode */
     EscapeData.hDevice = pOp->Device.hDevice;
     EscapeData.hContext = pOp->Context.hContext;
+#endif
     EscapeData.Type = D3DKMT_ESCAPE_DRIVERPRIVATE;
     EscapeData.Flags.HardwareAccess = 1;
     EscapeData.pPrivateDriverData = &EscData;
@@ -1333,6 +1336,28 @@ DWORD vboxDispIfCancelPendingResizeWDDM(PCVBOXDISPIF const pIf)
     return NO_ERROR;
 }
 
+static DWORD vboxDispIfWddmResizeDisplayVista(DEVMODE *paDeviceModes, DISPLAY_DEVICE *paDisplayDevices, DWORD cDevModes, UINT iChangedMode, BOOL fEnable, BOOL fExtDispSup)
+{
+    /* Without this, Windows will not ask the miniport for its
+     * mode table but uses an internal cache instead.
+     */
+    for (DWORD i = 0; i < cDevModes; i++)
+    {
+        DEVMODE tempDevMode;
+        ZeroMemory (&tempDevMode, sizeof (tempDevMode));
+        tempDevMode.dmSize = sizeof(DEVMODE);
+        EnumDisplaySettings((LPSTR)paDisplayDevices[i].DeviceName, 0xffffff, &tempDevMode);
+        Log(("VBoxTray: ResizeDisplayDevice: EnumDisplaySettings last error %d\n", GetLastError ()));
+    }
+
+    DWORD winEr = EnableAndResizeDispDev(paDeviceModes, paDisplayDevices, cDevModes, iChangedMode, paDeviceModes[iChangedMode].dmPelsWidth, paDeviceModes[iChangedMode].dmPelsHeight,
+            paDeviceModes[iChangedMode].dmBitsPerPel, paDeviceModes[iChangedMode].dmPosition.x, paDeviceModes[iChangedMode].dmPosition.y, fEnable, fExtDispSup);
+    if (winEr != NO_ERROR)
+        WARN(("VBoxTray: (WDDM) Failed EnableAndResizeDispDev winEr %d\n", winEr));
+
+    return winEr;
+}
+
 static DWORD vboxDispIfResizePerform(PCVBOXDISPIF const pIf, UINT iChangedMode, BOOL fEnable, BOOL fExtDispSup, DISPLAY_DEVICE *paDisplayDevices, DEVMODE *paDeviceModes, UINT cDevModes)
 {
     DWORD winEr;
@@ -1344,10 +1369,9 @@ static DWORD vboxDispIfResizePerform(PCVBOXDISPIF const pIf, UINT iChangedMode,
     }
     else
     {
-        winEr = EnableAndResizeDispDev(paDeviceModes, paDisplayDevices, cDevModes, iChangedMode, paDeviceModes[iChangedMode].dmPelsWidth, paDeviceModes[iChangedMode].dmPelsHeight,
-                paDeviceModes[iChangedMode].dmBitsPerPel, paDeviceModes[iChangedMode].dmPosition.x, paDeviceModes[iChangedMode].dmPosition.y, fEnable, fExtDispSup);
+        winEr = vboxDispIfWddmResizeDisplayVista(paDeviceModes, paDisplayDevices, cDevModes, iChangedMode, fEnable, fExtDispSup);
         if (winEr != NO_ERROR)
-            WARN(("VBoxTray: (WDDM) Failed EnableAndResizeDispDev winEr %d\n", winEr));
+            WARN(("VBoxTray: (WDDM) Failed vboxDispIfWddmResizeDisplayVista winEr %d\n", winEr));
     }
     return winEr;
 }
@@ -1691,8 +1715,11 @@ static DWORD vboxDispIfConfigureTargetsWDDM(VBOXDISPIF_OP *pOp, uint32_t *pcConn
 
     D3DKMT_ESCAPE EscapeData = {0};
     EscapeData.hAdapter = pOp->Adapter.hAdapter;
+#ifdef VBOX_DISPIF_WITH_OPCONTEXT
+    /* win8.1 does not allow context-based escapes for display-only mode */
     EscapeData.hDevice = pOp->Device.hDevice;
     EscapeData.hContext = pOp->Context.hContext;
+#endif
     EscapeData.Type = D3DKMT_ESCAPE_DRIVERPRIVATE;
     EscapeData.Flags.HardwareAccess = 1;
     EscapeData.pPrivateDriverData = &EscapeHdr;
@@ -1718,15 +1745,6 @@ static DWORD vboxDispIfResizeStartedWDDMOp(VBOXDISPIF_OP *pOp)
         return ERROR_GEN_FAILURE;
     }
 
-#ifdef DEBUG_misha
-    {
-        UINT cDcDisplays;
-        DWORD winEr2 = vboxDispIfWddmDcQueryNumDisplays(&cDcDisplays);
-        Assert(cDcDisplays == NumDevices);
-        /* testing */
-        Assert(cDcDisplays == 2);
-    }
-#endif
     DISPLAY_DEVICE *paDisplayDevices = (DISPLAY_DEVICE *)alloca (sizeof (DISPLAY_DEVICE) * NumDevices);
     DEVMODE *paDeviceModes = (DEVMODE *)alloca (sizeof (DEVMODE) * NumDevices);
     DWORD DevNum = 0;
@@ -1816,9 +1834,9 @@ static DWORD vboxDispIfResizeStartedWDDMOp(VBOXDISPIF_OP *pOp)
     {
         for (DWORD i = 0; i < cIds; ++i)
         {
-            winEr = EnableAndResizeDispDev(paNewDeviceModes, paNewDisplayDevices, NewDevNum, i, 0, 0, 0, 0, 0, FALSE, TRUE);
+            winEr = vboxDispIfWddmResizeDisplayVista(paNewDeviceModes, paNewDisplayDevices, NewDevNum, i, FALSE, TRUE);
             if (winEr != NO_ERROR)
-                WARN(("VBoxTray: vboxDispIfResizeStartedWDDMOp: EnableAndResizeDispDev failed winEr 0x%x\n", winEr));
+                WARN(("VBoxTray: vboxDispIfResizeStartedWDDMOp: vboxDispIfWddmResizeDisplayVista failed winEr 0x%x\n", winEr));
         }
     }
 
@@ -1994,13 +2012,16 @@ static DWORD vboxDispIfSeamlesCreateWDDM(PCVBOXDISPIF const pIf, VBOXDISPIF_SEAM
     HRESULT hr = vboxDispKmtOpenAdapter(&pIf->modeData.wddm.KmtCallbacks, &pSeamless->modeData.wddm.Adapter);
     if (SUCCEEDED(hr))
     {
+#ifdef VBOX_DISPIF_WITH_OPCONTEXT
         hr = vboxDispKmtCreateDevice(&pSeamless->modeData.wddm.Adapter, &pSeamless->modeData.wddm.Device);
         if (SUCCEEDED(hr))
         {
             hr = vboxDispKmtCreateContext(&pSeamless->modeData.wddm.Device, &pSeamless->modeData.wddm.Context, VBOXWDDM_CONTEXT_TYPE_CUSTOM_DISPIF_SEAMLESS,
                     0, 0, hEvent, 0ULL);
             if (SUCCEEDED(hr))
+#endif
                 return ERROR_SUCCESS;
+#ifdef VBOX_DISPIF_WITH_OPCONTEXT
             else
                 WARN(("VBoxTray: vboxDispKmtCreateContext failed hr 0x%x", hr));
 
@@ -2010,6 +2031,7 @@ static DWORD vboxDispIfSeamlesCreateWDDM(PCVBOXDISPIF const pIf, VBOXDISPIF_SEAM
             WARN(("VBoxTray: vboxDispKmtCreateDevice failed hr 0x%x", hr));
 
         vboxDispKmtCloseAdapter(&pSeamless->modeData.wddm.Adapter);
+#endif
     }
 
     return hr;
@@ -2017,8 +2039,10 @@ static DWORD vboxDispIfSeamlesCreateWDDM(PCVBOXDISPIF const pIf, VBOXDISPIF_SEAM
 
 static DWORD vboxDispIfSeamlesTermWDDM(VBOXDISPIF_SEAMLESS *pSeamless)
 {
+#ifdef VBOX_DISPIF_WITH_OPCONTEXT
     vboxDispKmtDestroyContext(&pSeamless->modeData.wddm.Context);
     vboxDispKmtDestroyDevice(&pSeamless->modeData.wddm.Device);
+#endif
     vboxDispKmtCloseAdapter(&pSeamless->modeData.wddm.Adapter);
 
     return NO_ERROR;
@@ -2028,10 +2052,12 @@ static DWORD vboxDispIfSeamlesSubmitWDDM(VBOXDISPIF_SEAMLESS *pSeamless, VBOXDIS
 {
     D3DKMT_ESCAPE EscapeData = {0};
     EscapeData.hAdapter = pSeamless->modeData.wddm.Adapter.hAdapter;
+#ifdef VBOX_DISPIF_WITH_OPCONTEXT
     EscapeData.hDevice = pSeamless->modeData.wddm.Device.hDevice;
     EscapeData.hContext = pSeamless->modeData.wddm.Context.hContext;
+#endif
     EscapeData.Type = D3DKMT_ESCAPE_DRIVERPRIVATE;
-    EscapeData.Flags.HardwareAccess = 1;
+    /*EscapeData.Flags.HardwareAccess = 1;*/
     EscapeData.pPrivateDriverData = pData;
     EscapeData.PrivateDriverDataSize = VBOXDISPIFESCAPE_SIZE(cbData);
 
