@@ -1,10 +1,10 @@
 #! /bin/sh
 #
-# Linux Additions X11 setup init script ($Revision: 80787 $)
+# Linux Additions X11 setup init script ($Revision: 91647 $)
 #
 
 #
-# Copyright (C) 2006-2010 Oracle Corporation
+# Copyright (C) 2006-2012 Oracle Corporation
 #
 # This file is part of VirtualBox Open Source Edition (OSE), as
 # available from http://www.virtualbox.org. This file is free software;
@@ -54,6 +54,7 @@ esac
 # The last of the three is for the X.org 6.7 included in Fedora Core 2
 xver=`X -version 2>&1`
 x_version=`echo "$xver" | sed -n 's/^X Window System Version \([0-9.]\+\)/\1/p'``echo "$xver" | sed -n 's/^XFree86 Version \([0-9.]\+\)/\1/p'``echo "$xver" | sed -n 's/^X Protocol Version 11, Revision 0, Release \([0-9.]\+\)/\1/p'``echo "$xver" | sed -n 's/^X.Org X Server \([0-9.]\+\)/\1/p'`
+x_version_short=`echo "${x_version}" | sed 's/\([0-9]*\.[0-9]*\)\..*/\1/'`
 # Version of Redhat or Fedora installed.  Needed for setting up selinux policy.
 redhat_release=`cat /etc/redhat-release 2> /dev/null`
 # All the different possible locations for XFree86/X.Org configuration files
@@ -312,7 +313,11 @@ setup()
     fi
 
     test -z "$x_version" -o -z "$modules_dir" &&
-        fail "Could not find the X.Org or XFree86 Window System."
+        {
+            echo
+            echo "Could not find the X.Org or XFree86 Window System, skipping."
+            exit 0
+        }
 
     echo
     # openSUSE 10.3 shipped X.Org 7.2 with X.Org Server 1.3, but didn't
@@ -328,16 +333,6 @@ setup()
             echo "installing the X.Org drivers."
             dox11config=""
             ;;
-        1.13.* )
-            xserver_version="X.Org Server 1.13"
-            vboxvideo_src=vboxvideo_drv_113.so
-            test "$system" = "redhat" || setupxorgconf=""
-            ;;
-        1.12.* )
-            xserver_version="X.Org Server 1.12"
-            vboxvideo_src=vboxvideo_drv_112.so
-            test "$system" = "redhat" || setupxorgconf=""
-            ;;
         1.11.* )
             xserver_version="X.Org Server 1.11"
             vboxvideo_src=vboxvideo_drv_111.so
@@ -351,7 +346,7 @@ setup()
         1.9.* )
             xserver_version="X.Org Server 1.9"
             vboxvideo_src=vboxvideo_drv_19.so
-            # Fedora 14 and later patched out vboxvideo detection
+            # Fedora 14 to 16 patched out vboxvideo detection
             test "$system" = "redhat" || setupxorgconf=""
             ;;
         1.8.* )
@@ -420,12 +415,21 @@ setup()
             automouse=""
             ;;
         * )
-            echo "Warning: unknown version of the X Window System installed.  Not installing"
-            echo "X Window System drivers."
-            dox11config=""
+            # Anything else, including all X server versions as of 1.12.
+            xserver_version="X.Org Server ${x_version_short}"
+            vboxvideo_src=vboxvideo_drv_`echo ${x_version_short} | sed 's/\.//'`.so
+            setupxorgconf=""
+            test -f "${lib_dir}/${vboxvideo_src}" ||
+            {
+                echo "Warning: unknown version of the X Window System installed.  Not installing"
+                echo "X Window System drivers."
+                dox11config=""
+                vboxvideo_src=""
+            }
             ;;
     esac
-    begin "Installing $xserver_version modules"
+    test -n "${dox11config}" &&
+        begin "Installing $xserver_version modules"
     rm "$modules_dir/drivers/vboxvideo_drv$driver_ext" 2>/dev/null
     rm "$modules_dir/input/vboxmouse_drv$driver_ext" 2>/dev/null
     case "$vboxvideo_src" in ?*)
