@@ -23,6 +23,9 @@
 #define __STDC_LIMIT_MACROS
 #define __STDC_CONSTANT_MACROS
 #include "slirp/libslirp.h"
+extern "C" {
+#include "slirp/slirp_dns.h"
+}
 #include "slirp/ctl.h"
 #include <VBox/vmm/pdmdrv.h>
 #include <VBox/vmm/pdmnetifs.h>
@@ -215,6 +218,10 @@ typedef DRVNAT *PDRVNAT;
 *   Internal Functions                                                         *
 *******************************************************************************/
 static void drvNATNotifyNATThread(PDRVNAT pThis, const char *pszWho);
+DECLINLINE(void) drvNATHostNetworkConfigurationChangeEventStrategySelector(
+  PDRVNAT pThis,
+  bool fHostNetworkConfigurationEventListener);
+static DECLCALLBACK(int) drvNATReinitializeHostNameResolving(PDRVNAT pThis);
 
 
 static DECLCALLBACK(int) drvNATRecv(PPDMDRVINS pDrvIns, PPDMTHREAD pThread)
@@ -962,7 +969,17 @@ static DECLCALLBACK(void) drvNatDnsChanged(SCDynamicStoreRef hDynStor, CFArrayRe
 {
     PDRVNAT pThis = (PDRVNAT)pvUser;
 
-    pThis->pIAboveConfig->pfnSetLinkState(pThis->pIAboveConfig, PDMNETWORKLINKSTATE_DOWN_RESUME);
+    LogRel(("NAT: DNS servers changed, triggering reconnect\n"));
+
+    CFDictionaryRef hDnsDict = (CFDictionaryRef)SCDynamicStoreCopyValue(hDynStor, CFSTR("State:/Network/Global/DNS"));
+    if (hDnsDict)
+    {
+        CFArrayRef hArrAddresses = (CFArrayRef)CFDictionaryGetValue(hDnsDict, kSCPropNetDNSServerAddresses);
+        if (hArrAddresses)
+            drvNATHostNetworkConfigurationChangeEventStrategySelector(pThis, /* RT_OS_DARWIN */ true);
+
+        CFRelease(hDnsDict);
+    }
 }
 #endif
 
@@ -1032,13 +1049,10 @@ static DECLCALLBACK(void) drvNATResume(PPDMDRVINS pDrvIns)
     switch (enmReason)
     {
         case VMRESUMEREASON_HOST_RESUME:
-            /*
-             * Host resumed from a suspend and the network might have changed.
-             * Disconnect the guest from the network temporarily to let it pick up the changes.
-             */
-#ifndef RT_OS_DARWIN
-            pThis->pIAboveConfig->pfnSetLinkState(pThis->pIAboveConfig, 
-                                                  PDMNETWORKLINKSTATE_DOWN_RESUME);
+#if RT_OS_DARWIN
+            drvNATHostNetworkConfigurationChangeEventStrategySelector(pThis, false);
+#else
+            drvNATHostNetworkConfigurationChangeEventStrategySelector(pThis, true);
 #endif
             return;
         default: /* Ignore every other resume reason. */
@@ -1048,6 +1062,63 @@ static DECLCALLBACK(void) drvNATResume(PPDMDRVINS pDrvIns)
 }
 
 
+static DECLCALLBACK(int) drvNATReinitializeHostNameResolving(PDRVNAT pThis)
+{
+    slirpReleaseDnsSettings(pThis->pNATState);
+    slirpInitializeDnsSettings(pThis->pNATState);
+    return VINF_SUCCESS;
+}
+
+/**
+ * This function at this stage could be called from two places, but both from non-NAT thread,
+ * - drvNATResume (EMT?)
+ * - drvNatDnsChanged (darwin, GUI or main) "listener"
+ * Thread here is important, because we need to change DNS server list and domain name (+ perhaps, 
+ * search string) at runtime (VBOX_NAT_ENFORCE_INTERNAL_DNS_UPDATE).
+ * XXX: for 4.2 we haven't got API for receiving Host Network changes updates, so here should invent other way to follow. 
+ */
+DECLINLINE(void) drvNATHostNetworkConfigurationChangeEventStrategySelector(PDRVNAT pThis,
+                                                                        bool fHostNetworkConfigurationEventListener)
+{
+    int strategy = slirp_host_network_configuration_change_strategy_selector(pThis->pNATState);
+    switch (strategy)
+    {
+                 
+        case VBOX_NAT_HNCE_DNSPROXY:
+            {
+                /**
+                 * It's unsafe to to do it directly on non-NAT thread
+                 * so we schedule the worker and kick the NAT thread. 
+                 */
+                RTREQQUEUE hQueue = pThis->hSlirpReqQueue;
+                
+                int rc = RTReqQueueCallEx(hQueue, NULL /*ppReq*/, 0 /*cMillies*/, 
+                                          RTREQFLAGS_VOID | RTREQFLAGS_NO_WAIT,
+                                          (PFNRT)drvNATReinitializeHostNameResolving, 1, pThis);
+                if (RT_SUCCESS(rc))
+                    drvNATNotifyNATThread(pThis, "drvNATNetworkUp_SendBuf");
+
+
+                return;
+
+            }
+        case VBOX_NAT_HNCE_EXSPOSED_NAME_RESOLUTION_INFO:
+            /*
+             * Host resumed from a suspend and the network might have changed.
+             * Disconnect the guest from the network temporarily to let it pick up the changes.
+             */
+
+            if (fHostNetworkConfigurationEventListener)
+                pThis->pIAboveConfig->pfnSetLinkState(pThis->pIAboveConfig,
+                                                      PDMNETWORKLINKSTATE_DOWN_RESUME);
+            return;
+        case VBOX_NAT_HNCE_HOSTRESOLVER:
+        default:
+            return;
+    }
+}
+
+
 /**
  * Info handler.
  */
