@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2011 Oracle Corporation
+ * Copyright (C) 2006-2012 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -303,9 +303,17 @@ int slirp_init(PNATState *ppData, uint32_t u32NetAddr, uint32_t u32Netmask,
         return VERR_NO_MEMORY;
     pData->fPassDomain = !fUseHostResolver ? fPassDomain : false;
     pData->fUseHostResolver = fUseHostResolver;
+    pData->fUseHostResolverPermanent = fUseHostResolver;
     pData->pvUser = pvUser;
     pData->netmask = u32Netmask;
 
+    rc = RTSemRWCreate(&pData->hSemRwHandlerChain);
+    if (RT_FAILURE(rc))
+    {
+        RTMemFree(pData);
+        return rc;
+    }
+
     /* sockets & TCP defaults */
     pData->socket_rcv = 64 * _1K;
     pData->socket_snd = 64 * _1K;
@@ -547,6 +555,7 @@ void slirp_term(PNATState pData)
          "\n"));
 #endif
 #endif
+    RTSemRWDestroy(pData->hSemRwHandlerChain);
     RTMemFree(pData);
 }
 
@@ -2011,3 +2020,11 @@ void slirp_info(PNATState pData, PCDBGFINFOHLP pHlp, const char *pszArgs)
                         rule->activated ? ' ' : '*');
     }
 }
+
+
+int slirp_host_network_configuration_change_strategy_selector(const PNATState pData)
+{
+    if (pData->fUseHostResolver) return VBOX_NAT_HNCE_HOSTRESOLVER;
+    if (pData->fUseDnsProxy) return VBOX_NAT_HNCE_DNSPROXY;
+    return VBOX_NAT_HNCE_EXSPOSED_NAME_RESOLUTION_INFO;
+}
