@@ -9,7 +9,7 @@
  */
 
 /*
- * Copyright (C) 2006-2011 Oracle Corporation
+ * Copyright (C) 2006-2012 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -1161,7 +1161,7 @@ DECLCALLBACK(int) VMPowerUpThread(RTTHREAD Thread, void *pvUser)
             endProgressInfo();
             if (RT_SUCCESS(rc))
             {
-                rc = VMR3Resume(gpVM);
+                rc = VMR3Resume(gpVM, VMRESUMEREASON_STATE_RESTORED);
                 AssertRC(rc);
                 gDisplay->setRunning();
             }
