@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2012 Oracle Corporation
+ * Copyright (C) 2006-2013 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -133,9 +133,9 @@ void printUsage(USAGECATEGORY u64Cmd, PRTSTREAM pStrm)
 
     if (u64Cmd & USAGE_SHOWVMINFO)
         RTStrmPrintf(pStrm,
-                           "%s showvminfo %s      <uuid>|<name> [--details]\n"
+                           "%s showvminfo %s      <uuid|vmname> [--details]\n"
                      "                            [--machinereadable]\n"
-                           "%s showvminfo %s      <uuid>|<name> --log <idx>\n"
+                           "%s showvminfo %s      <uuid|vmname> --log <idx>\n"
                      "\n", SEP, SEP);
 
     if (u64Cmd & USAGE_REGISTERVM)
@@ -145,7 +145,7 @@ void printUsage(USAGECATEGORY u64Cmd, PRTSTREAM pStrm)
 
     if (u64Cmd & USAGE_UNREGISTERVM)
         RTStrmPrintf(pStrm,
-                           "%s unregistervm %s    <uuid>|<name> [--delete]\n"
+                           "%s unregistervm %s    <uuid|vmname> [--delete]\n"
                      "\n", SEP);
 
     if (u64Cmd & USAGE_CREATEVM)
@@ -161,7 +161,7 @@ void printUsage(USAGECATEGORY u64Cmd, PRTSTREAM pStrm)
     if (u64Cmd & USAGE_MODIFYVM)
     {
         RTStrmPrintf(pStrm,
-                           "%s modifyvm %s        <uuid|name>\n"
+                           "%s modifyvm %s        <uuid|vmname>\n"
                      "                            [--name <name>]\n"
                      "                            [--groups <group>, ...]\n"
                      "                            [--ostype <ostype>]\n"
@@ -267,7 +267,6 @@ void printUsage(USAGECATEGORY u64Cmd, PRTSTREAM pStrm)
                      "                            [--lptmode<1-N> <devicename>]\n"
 #endif
                      "                            [--guestmemoryballoon <balloonsize in MB>]\n"
-                     "                            [--gueststatisticsinterval <seconds>]\n"
                      "                            [--audio none|null", SEP);
         if (fWin)
         {
@@ -355,6 +354,9 @@ void printUsage(USAGECATEGORY u64Cmd, PRTSTREAM pStrm)
                      "                            [--faulttolerancesyncinterval <msec>]\n"
                      "                            [--faulttolerancepassword <password>]\n"
 #endif
+#ifdef VBOX_WITH_USB_VIDEO
+                     "                            [--usbwebcam on|off]\n"
+#endif
 #ifdef VBOX_WITH_USB_CARDREADER
                      "                            [--usbcardreader on|off]\n"
 #endif
@@ -369,7 +371,7 @@ void printUsage(USAGECATEGORY u64Cmd, PRTSTREAM pStrm)
 
     if (u64Cmd & USAGE_CLONEVM)
         RTStrmPrintf(pStrm,
-                           "%s clonevm %s         <uuid>|<name>\n"
+                           "%s clonevm %s         <uuid|vmname>\n"
                      "                            [--snapshot <uuid>|<name>]\n"
                      "                            [--mode machine|machineandchildren|all]\n"
                      "                            [--options link|keepallmacs|keepnatmacs|\n"
@@ -383,7 +385,7 @@ void printUsage(USAGECATEGORY u64Cmd, PRTSTREAM pStrm)
 
     if (u64Cmd & USAGE_IMPORTAPPLIANCE)
         RTStrmPrintf(pStrm,
-                           "%s import %s          <ovf/ova>\n"
+                           "%s import %s          <ovfname/ovaname>\n"
                      "                            [--dry-run|-n]\n"
                      "                            [--options keepallmacs|keepnatmacs]\n"
                      "                            [more options]\n"
@@ -408,7 +410,7 @@ void printUsage(USAGECATEGORY u64Cmd, PRTSTREAM pStrm)
     if (u64Cmd & USAGE_STARTVM)
     {
         RTStrmPrintf(pStrm,
-                           "%s startvm %s         <uuid>|<name>...\n"
+                           "%s startvm %s         <uuid|vmname>...\n"
                      "                            [--type gui", SEP);
         if (fVBoxSDL)
             RTStrmPrintf(pStrm, "|sdl");
@@ -420,7 +422,7 @@ void printUsage(USAGECATEGORY u64Cmd, PRTSTREAM pStrm)
     if (u64Cmd & USAGE_CONTROLVM)
     {
         RTStrmPrintf(pStrm,
-                           "%s controlvm %s       <uuid>|<name>\n"
+                           "%s controlvm %s       <uuid|vmname>\n"
                      "                            pause|resume|reset|poweroff|savestate|\n"
                      "                            acpipowerbutton|acpisleepbutton|\n"
                      "                            keyboardputscancode <hex> [<hex> ...]|\n"
@@ -433,25 +435,25 @@ void printUsage(USAGECATEGORY u64Cmd, PRTSTREAM pStrm)
                      "                            nic<1-N> null|nat|bridged|intnet|generic\n"
                      "                                     [<devicename>] |\n"
 #endif /* !VBOX_WITH_NETFLT */
-                     "                            nictrace<1-N> on|off\n"
-                     "                            nictracefile<1-N> <filename>\n"
-                     "                            nicproperty<1-N> name=[value]\n"
+                     "                            nictrace<1-N> on|off |\n"
+                     "                            nictracefile<1-N> <filename> |\n"
+                     "                            nicproperty<1-N> name=[value] |\n"
+                     "                            nicpromisc<1-N> deny|allow-vms|allow-all |\n"
                      "                            natpf<1-N> [<rulename>],tcp|udp,[<hostip>],\n"
-                     "                                          <hostport>,[<guestip>],<guestport>\n"
-                     "                            natpf<1-N> delete <rulename>\n"
-                     "                            guestmemoryballoon <balloonsize in MB>]\n"
-                     "                            gueststatisticsinterval <seconds>]\n"
+                     "                                        <hostport>,[<guestip>],<guestport> |\n"
+                     "                            natpf<1-N> delete <rulename> |\n"
+                     "                            guestmemoryballoon <balloonsize in MB> |\n"
                      "                            usbattach <uuid>|<address> |\n"
                      "                            usbdetach <uuid>|<address> |\n"
                      "                            clipboard disabled|hosttoguest|guesttohost|\n"
-                     "                                         bidirectional]\n"
-                     "                            draganddrop disabled|hosttoguest]\n"
+                     "                                      bidirectional |\n"
+                     "                            draganddrop disabled|hosttoguest |\n"
                      "                            vrde on|off |\n"
                      "                            vrdeport <port> |\n"
                      "                            vrdeproperty <name=[value]> |\n"
-                     "                            vrdevideochannelquality <percent>\n"
+                     "                            vrdevideochannelquality <percent> |\n"
                      "                            setvideomodehint <xres> <yres> <bpp>\n"
-                     "                                            [[<display>] [<enabled:yes|no>\n"
+                     "                                            [[<display>] [<enabled:yes|no> |\n"
                      "                                              [<xorigin> <yorigin>]]] |\n"
                      "                            screenshotpng <file> [display] |\n"
                      "                            setcredentials <username>\n"
@@ -461,40 +463,40 @@ void printUsage(USAGECATEGORY u64Cmd, PRTSTREAM pStrm)
                      "                            teleport --host <name> --port <port>\n"
                      "                                     [--maxdowntime <msec>]\n"
                      "                                     [--passwordfile <file> |\n"
-                     "                                      --password <password>]\n"
-                     "                            plugcpu <id>\n"
-                     "                            unplugcpu <id>\n"
+                     "                                      --password <password>] |\n"
+                     "                            plugcpu <id> |\n"
+                     "                            unplugcpu <id> |\n"
                      "                            cpuexecutioncap <1-100>\n"
                      "\n", SEP);
     }
 
     if (u64Cmd & USAGE_DISCARDSTATE)
         RTStrmPrintf(pStrm,
-                           "%s discardstate %s    <uuid>|<name>\n"
+                           "%s discardstate %s    <uuid|vmname>\n"
                      "\n", SEP);
 
     if (u64Cmd & USAGE_ADOPTSTATE)
         RTStrmPrintf(pStrm,
-                           "%s adoptstate %s      <uuid>|<name> <state_file>\n"
+                           "%s adoptstate %s      <uuid|vmname> <state_file>\n"
                      "\n", SEP);
 
     if (u64Cmd & USAGE_SNAPSHOT)
         RTStrmPrintf(pStrm,
-                           "%s snapshot %s        <uuid>|<name>\n"
-                     "                            take <name> [--description <desc>] [--pause] |\n"
-                     "                            delete <uuid>|<name> |\n"
-                     "                            restore <uuid>|<name> |\n"
+                           "%s snapshot %s        <uuid|vmname>\n"
+                     "                            take <name> [--description <desc>] [--live] |\n"
+                     "                            delete <uuid|snapname> |\n"
+                     "                            restore <uuid|snapname> |\n"
                      "                            restorecurrent |\n"
-                     "                            edit <uuid>|<name>|--current\n"
+                     "                            edit <uuid|snapname>|--current\n"
                      "                                 [--name <name>]\n"
                      "                                 [--description <desc>] |\n"
                      "                            list [--details|--machinereadable]\n"
-                     "                            showvminfo <uuid>|<name>\n"
+                     "                            showvminfo <uuid|snapname>\n"
                      "\n", SEP);
 
     if (u64Cmd & USAGE_CLOSEMEDIUM)
         RTStrmPrintf(pStrm,
-                           "%s closemedium %s     disk|dvd|floppy <uuid>|<filename>\n"
+                           "%s closemedium %s     disk|dvd|floppy <uuid|filename>\n"
                      "                            [--delete]\n"
                      "\n", SEP);
 
@@ -506,7 +508,7 @@ void printUsage(USAGECATEGORY u64Cmd, PRTSTREAM pStrm)
                      "                            [--device <number>]\n"
                      "                            [--type dvddrive|hdd|fdd]\n"
                      "                            [--medium none|emptydrive|additions|\n"
-                     "                                      <uuid>|<filename>|host:<drive>|iscsi]\n"
+                     "                                      <uuid|filename>|host:<drive>|iscsi]\n"
                      "                            [--mtype normal|writethrough|immutable|shareable|\n"
                      "                                     readonly|multiattach]\n"
                      "                            [--comment <text>]\n"
@@ -557,7 +559,7 @@ void printUsage(USAGECATEGORY u64Cmd, PRTSTREAM pStrm)
 
     if (u64Cmd & USAGE_SHOWHDINFO)
         RTStrmPrintf(pStrm,
-                           "%s showhdinfo %s      <uuid>|<filename>\n"
+                           "%s showhdinfo %s      <uuid|filename>\n"
                      "\n", SEP);
 
     if (u64Cmd & USAGE_CREATEHD)
@@ -571,7 +573,7 @@ void printUsage(USAGECATEGORY u64Cmd, PRTSTREAM pStrm)
 
     if (u64Cmd & USAGE_MODIFYHD)
         RTStrmPrintf(pStrm,
-                           "%s modifyhd %s        <uuid>|<filename>\n"
+                           "%s modifyhd %s        <uuid|filename>\n"
                      "                            [--type normal|writethrough|immutable|shareable|\n"
                      "                                    readonly|multiattach]\n"
                      "                            [--autoreset on|off]\n"
@@ -581,7 +583,7 @@ void printUsage(USAGECATEGORY u64Cmd, PRTSTREAM pStrm)
 
     if (u64Cmd & USAGE_CLONEHD)
         RTStrmPrintf(pStrm,
-                           "%s clonehd %s         <uuid>|<filename> <uuid>|<outputfile>\n"
+                           "%s clonehd %s         <uuid|inputfile> <uuid|outputfile>\n"
                      "                            [--format VDI|VMDK|VHD|RAW|<other>]\n"
                      "                            [--variant Standard,Fixed,Split2G,Stream,ESX]\n"
                      "                            [--existing]\n"
@@ -601,13 +603,13 @@ void printUsage(USAGECATEGORY u64Cmd, PRTSTREAM pStrm)
 
     if (u64Cmd & USAGE_GETEXTRADATA)
         RTStrmPrintf(pStrm,
-                           "%s getextradata %s    global|<uuid>|<name>\n"
+                           "%s getextradata %s    global|<uuid|vmname>\n"
                      "                            <key>|enumerate\n"
                      "\n", SEP);
 
     if (u64Cmd & USAGE_SETEXTRADATA)
         RTStrmPrintf(pStrm,
-                           "%s setextradata %s    global|<uuid>|<name>\n"
+                           "%s setextradata %s    global|<uuid|vmname>\n"
                      "                            <key>\n"
                      "                            [<value>] (no value deletes key)\n"
                      "\n", SEP);
@@ -625,7 +627,7 @@ void printUsage(USAGECATEGORY u64Cmd, PRTSTREAM pStrm)
     if (u64Cmd & USAGE_USBFILTER_ADD)
         RTStrmPrintf(pStrm,
                            "%s usbfilter %s       add <index,0-N>\n"
-                     "                            --target <uuid>|<name>|global\n"
+                     "                            --target <uuid|vmname>|global\n"
                      "                            --name <string>\n"
                      "                            --action ignore|hold (global filters only)\n"
                      "                            [--active yes|no] (yes)\n"
@@ -642,7 +644,7 @@ void printUsage(USAGECATEGORY u64Cmd, PRTSTREAM pStrm)
     if (u64Cmd & USAGE_USBFILTER_MODIFY)
         RTStrmPrintf(pStrm,
                            "%s usbfilter %s       modify <index,0-N>\n"
-                     "                            --target <uuid>|<name>|global\n"
+                     "                            --target <uuid|vmname>|global\n"
                      "                            [--name <string>]\n"
                      "                            [--action ignore|hold] (global filters only)\n"
                      "                            [--active yes|no]\n"
@@ -659,19 +661,19 @@ void printUsage(USAGECATEGORY u64Cmd, PRTSTREAM pStrm)
     if (u64Cmd & USAGE_USBFILTER_REMOVE)
         RTStrmPrintf(pStrm,
                            "%s usbfilter %s       remove <index,0-N>\n"
-                     "                            --target <uuid>|<name>|global\n"
+                     "                            --target <uuid|vmname>|global\n"
                      "\n", SEP);
 
     if (u64Cmd & USAGE_SHAREDFOLDER_ADD)
         RTStrmPrintf(pStrm,
-                           "%s sharedfolder %s    add <vmname>|<uuid>\n"
+                           "%s sharedfolder %s    add <uuid|vmname>\n"
                      "                            --name <name> --hostpath <hostpath>\n"
                      "                            [--transient] [--readonly] [--automount]\n"
                      "\n", SEP);
 
     if (u64Cmd & USAGE_SHAREDFOLDER_REMOVE)
         RTStrmPrintf(pStrm,
-                           "%s sharedfolder %s    remove <vmname>|<uuid>\n"
+                           "%s sharedfolder %s    remove <uuid|vmname>\n"
                      "                            --name <name> [--transient]\n"
                      "\n", SEP);
 
@@ -688,7 +690,7 @@ void printUsage(USAGECATEGORY u64Cmd, PRTSTREAM pStrm)
     if (u64Cmd & USAGE_DEBUGVM)
     {
         RTStrmPrintf(pStrm,
-                           "%s debugvm %s         <uuid>|<name>\n"
+                           "%s debugvm %s         <uuid|vmname>\n"
                      "                            dumpguestcore --filename <name> |\n"
                      "                            info <item> [args] |\n"
                      "                            injectnmi |\n"
