@@ -5,7 +5,7 @@
  */
 
 /*
- * Copyright (C) 2006-2007 Oracle Corporation
+ * Copyright (C) 2006-2012 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -35,16 +35,23 @@
 /* COM includes: */
 #include "COMEnums.h"
 
+#include "CDisplay.h"
+
 /* Other VBox includes: */
 #include <iprt/assert.h>
 #include <iprt/critsect.h>
 #include <iprt/asm.h>
 #include <iprt/err.h>
+#include <iprt/list.h>
 #include <VBox/VBoxGL2D.h>
 #ifdef VBOXVHWA_PROFILE_FPS
 # include <iprt/stream.h>
 #endif /* VBOXVHWA_PROFILE_FPS */
 
+#ifndef S_FALSE
+# define S_FALSE ((HRESULT)1L)
+#endif
+
 #ifdef DEBUG_misha
 # define VBOXVHWA_PROFILE_FPS
 #endif /* DEBUG_misha */
@@ -1008,6 +1015,18 @@ class VBoxVHWASurfList
 public:
 
     VBoxVHWASurfList() : mCurrent(NULL) {}
+
+    void moveTo(VBoxVHWASurfList *pDst)
+    {
+        for (SurfList::iterator it = mSurfaces.begin();
+             it != mSurfaces.end(); it = mSurfaces.begin())
+        {
+            pDst->add((*it));
+        }
+
+        Assert(empty());
+    }
+
     void add(VBoxVHWASurfaceBase *pSurf)
     {
         VBoxVHWASurfList * pOld = pSurf->getComplexList();
@@ -1018,7 +1037,7 @@ public:
         mSurfaces.push_back(pSurf);
         pSurf->setComplexList(this);
     }
-
+/*
     void clear()
     {
         for (SurfList::iterator it = mSurfaces.begin();
@@ -1029,7 +1048,7 @@ public:
         mSurfaces.clear();
         mCurrent = NULL;
     }
-
+*/
     size_t size() const {return mSurfaces.size(); }
 
     void remove(VBoxVHWASurfaceBase *pSurf)
@@ -1070,7 +1089,12 @@ public:
     {
         VBoxVHWASurfaceBase * old = mSurfVGA;
         mSurfVGA = pVga;
-        mPrimary.clear();
+        if (!mPrimary.empty())
+        {
+            VBoxVHWASurfList *pNewList = new VBoxVHWASurfList();
+            mPrimary.moveTo(pNewList);
+            Assert(mPrimary.empty());
+        }
         if(pVga)
         {
             Assert(!pVga->getComplexList());
@@ -1188,10 +1212,6 @@ typedef struct VBOXVHWAFUNCCALLBACKINFO
 class VBoxVHWACommandElement
 {
 public:
-    VBoxVHWACommandElement() :
-        bNewEvent(false)
-    {}
-
     void setVHWACmd(struct VBOXVHWACMD * pCmd)
     {
         mType = VBOXVHWA_PIPECMD_VHWA;
@@ -1229,15 +1249,12 @@ public:
         }
     }
 
-    void setNewEvent(bool bNew) {bNewEvent = bNew;}
-    bool isNewEvent() const { return bNewEvent; }
-
     VBOXVHWA_PIPECMD_TYPE type() const {return mType;}
     const QRect & rect() const {return mRect;}
     struct VBOXVHWACMD * vhwaCmd() const {return u.mpCmd;}
     const VBOXVHWAFUNCCALLBACKINFO & func() const {return u.mFuncCallback; }
 
-    VBoxVHWACommandElement * mpNext;
+    RTLISTNODE ListNode;
 private:
     VBOXVHWA_PIPECMD_TYPE mType;
     union
@@ -1246,141 +1263,6 @@ private:
         VBOXVHWAFUNCCALLBACKINFO mFuncCallback;
     }u;
     QRect                 mRect;
-    bool bNewEvent;
-};
-
-class VBoxVHWACommandElementPipe
-{
-public:
-    VBoxVHWACommandElementPipe() :
-        mpFirst(NULL),
-        mpLast(NULL)
-    {}
-
-    void put(VBoxVHWACommandElement *pCmd)
-    {
-        if (mpLast)
-        {
-            Assert(mpFirst);
-            mpLast->mpNext = pCmd;
-            mpLast = pCmd;
-        }
-        else
-        {
-            Assert(!mpFirst);
-            mpFirst = pCmd;
-            mpLast = pCmd;
-        }
-        pCmd->mpNext= NULL;
-
-    }
-
-    void setFrom(VBoxVHWACommandElementPipe *pOther)
-    {
-        mpFirst = pOther->detachList(&mpLast);
-    }
-
-    void set(VBoxVHWACommandElement *pFirst, VBoxVHWACommandElement *pLast)
-    {
-        mpFirst = pFirst;
-        mpLast = pLast;
-        if (mpLast)
-            mpLast->mpNext = NULL;
-    }
-
-    void prepend(VBoxVHWACommandElement *pFirst, VBoxVHWACommandElement *pLast)
-    {
-        if (!mpFirst)
-            set(pFirst, pLast);
-        else if (pLast)
-        {
-            pLast->mpNext = mpFirst;
-            mpFirst = pFirst;
-        }
-    }
-
-    void prependFrom(VBoxVHWACommandElementPipe *pOther)
-    {
-        VBoxVHWACommandElement *pFirst;
-        VBoxVHWACommandElement *pLast;
-        pFirst = pOther->detachList(&pLast);
-        prepend(pFirst, pLast);
-    }
-
-    void append(VBoxVHWACommandElement *pFirst, VBoxVHWACommandElement *pLast)
-    {
-        if (!mpLast)
-            set(pFirst, pLast);
-        else if (pLast)
-        {
-            mpLast->mpNext = pFirst;
-            mpLast = pLast;
-            pLast->mpNext = NULL;
-        }
-    }
-
-    VBoxVHWACommandElement * detachList(VBoxVHWACommandElement **ppLast)
-    {
-        if (mpLast)
-        {
-            VBoxVHWACommandElement * pHead = mpFirst;
-            if (ppLast)
-                *ppLast = mpLast;
-            mpFirst = NULL;
-            mpLast = NULL;
-            return pHead;
-        }
-        if (ppLast)
-            *ppLast = NULL;
-        return NULL;
-    }
-
-
-
-    const VBoxVHWACommandElement * contentsRo (const VBoxVHWACommandElement **ppLast) const
-    {
-        if (ppLast)
-            *ppLast = mpLast;
-        return mpFirst;
-    }
-
-    bool isEmpty() const { return !mpLast; }
-
-private:
-    VBoxVHWACommandElement *mpFirst;
-    VBoxVHWACommandElement *mpLast;
-};
-
-class VBoxVHWACommandElementStack
-{
-public:
-    VBoxVHWACommandElementStack() :
-        mpFirst(NULL) {}
-
-    void push(VBoxVHWACommandElement *pCmd)
-    {
-        pCmd->mpNext = mpFirst;
-        mpFirst = pCmd;
-    }
-
-    void pusha(VBoxVHWACommandElement *pFirst, VBoxVHWACommandElement *pLast)
-    {
-        pLast->mpNext = mpFirst;
-        mpFirst = pFirst;
-    }
-
-    VBoxVHWACommandElement * pop()
-    {
-        if(mpFirst)
-        {
-            VBoxVHWACommandElement * ret = mpFirst;
-            mpFirst = ret->mpNext;
-            return ret;
-        }
-        return NULL;
-    }
-private:
-    VBoxVHWACommandElement *mpFirst;
 };
 
 class VBoxVHWARefCounter
@@ -1424,20 +1306,15 @@ private:
     volatile uint32_t m_cRefs;
 };
 
-#define VBOXVHWACMDPIPEC_NEWEVENT      0x00000001
-#define VBOXVHWACMDPIPEC_COMPLETEEVENT 0x00000002
 class VBoxVHWACommandElementProcessor
 {
 public:
     VBoxVHWACommandElementProcessor(QObject *pNotifyObject);
     ~VBoxVHWACommandElementProcessor();
-    void postCmd(VBOXVHWA_PIPECMD_TYPE aType, void * pvData, uint32_t flags);
-    bool completeCurrentEvent();
-    class VBoxVHWACommandElement * detachCmdList(class VBoxVHWACommandElement ** ppLast,
-            class VBoxVHWACommandElement * pFirst2Free, VBoxVHWACommandElement * pLast2Free);
-    void putBack(class VBoxVHWACommandElement * pFirst2Put, VBoxVHWACommandElement * pLast2Put,
-            class VBoxVHWACommandElement * pFirst2Free, VBoxVHWACommandElement * pLast2Free);
-    void reset(class VBoxVHWACommandElement ** ppHead, class VBoxVHWACommandElement ** ppTail);
+    void postCmd(VBOXVHWA_PIPECMD_TYPE aType, void * pvData);
+    VBoxVHWACommandElement *getCmd();
+    void doneCmd();
+    void reset(CDisplay *pDisplay);
     void setNotifyObject(QObject *pNotifyObject);
     int loadExec (struct SSMHANDLE * pSSM, uint32_t u32Version, void *pvVRAM);
     void saveExec (struct SSMHANDLE * pSSM, void *pvVRAM);
@@ -1445,19 +1322,14 @@ public:
     void enable();
     void lock();
     void unlock();
-#ifdef DEBUG_misha
-    void checkConsistence(uint32_t cEvents2Submit = 0, const VBoxVHWACommandElementPipe *pPipe = NULL);
-#endif
 private:
     RTCRITSECT mCritSect;
-    VBoxVHWACommandElementPipe m_CmdPipe;
+    RTLISTNODE mCommandList;
     QObject *m_pNotifyObject;
     VBoxVHWARefCounter m_NotifyObjectRefs;
-    bool mbNewEvent;
-    bool mbProcessingList;
+    VBoxVHWACommandElement *mpCurCmd;
+    bool mbResetting;
     uint32_t mcDisabled;
-    VBoxVHWACommandElementStack mFreeElements;
-    VBoxVHWACommandElement mElementsBuffer[2048];
 };
 
 /* added to workaround this ** [VBox|UI] duplication */
@@ -1835,15 +1707,10 @@ public:
         Q_UNUSED(uBytesPerLine);
         Q_UNUSED(uWidth);
         Q_UNUSED(uHeight);
+        Q_UNUSED(pResult);
         Q_UNUSED(pbFinished);
 
-        if (mCmdPipe.completeCurrentEvent())
-            return false;
-
-        /* TODO: more graceful resize handling */
-        *pResult = E_FAIL;
-
-        return true;
+        return false;
     }
 
     void onResizeEventPostprocess (const VBoxFBSizeInfo &re, const QPoint & topLeft);
@@ -1921,7 +1788,7 @@ private:
     void vboxDoVHWACmd (void *cmd);
     void addMainDirtyRect (const QRect & aRect);
     void vboxCheckUpdateOverlay (const QRect & rect);
-    VBoxVHWACommandElement * processCmdList (VBoxVHWACommandElement * pCmd, bool bFirst);
+    void processCmd (VBoxVHWACommandElement * pCmd);
 
     int vhwaConstruct (struct VBOXVHWACMD_HH_CONSTRUCT *pCmd);
 
@@ -1979,7 +1846,17 @@ public:
 
     STDMETHOD(ProcessVHWACommand)(BYTE *pCommand)
     {
-        return mOverlay.onVHWACommand ((struct VBOXVHWACMD*)pCommand);
+        int rc;
+        T::lock();
+        rc = mOverlay.onVHWACommand ((struct VBOXVHWACMD*)pCommand);
+        T::unlock();
+        if (rc == VINF_CALLBACK_RETURN)
+            return S_OK;
+        else if (RT_SUCCESS(rc))
+            return S_FALSE;
+        else if (rc == VERR_INVALID_STATE)
+            return E_ACCESSDENIED;
+        return E_FAIL;
     }
 
     void doProcessVHWACommand (QEvent * pEvent)
@@ -1992,27 +1869,32 @@ public:
                               ULONG aWidth, ULONG aHeight,
                               BOOL *aFinished)
    {
-        HRESULT result;
-        if (mOverlay.onRequestResize (aScreenId, aPixelFormat,
+        HRESULT hr;
+        T::lock();
+        if (!mOverlay.onRequestResize (aScreenId, aPixelFormat,
                 aVRAM, aBitsPerPixel, aBytesPerLine,
                 aWidth, aHeight,
-                &result,
+                &hr,
                 aFinished))
         {
-            return result;
+            hr = T::RequestResize (aScreenId, aPixelFormat,
+                    aVRAM, aBitsPerPixel, aBytesPerLine,
+                    aWidth, aHeight,
+                    aFinished);
         }
-        return T::RequestResize (aScreenId, aPixelFormat,
-                aVRAM, aBitsPerPixel, aBytesPerLine,
-                aWidth, aHeight,
-                aFinished);
+        T::unlock();
+        return hr;
    }
 
     STDMETHOD(NotifyUpdate) (ULONG aX, ULONG aY,
                              ULONG aW, ULONG aH)
     {
-        if (mOverlay.onNotifyUpdate (aX, aY, aW, aH))
-            return S_OK;
-        return T::NotifyUpdate (aX, aY, aW, aH);
+        HRESULT hr = S_OK;
+        T::lock();
+        if (!mOverlay.onNotifyUpdate (aX, aY, aW, aH))
+            hr = T::NotifyUpdate (aX, aY, aW, aH);
+        T::unlock();
+        return hr;
     }
 
     STDMETHOD(VideoModeSupported) (ULONG uWidth, ULONG uHeight, ULONG uBPP,
@@ -2023,16 +1905,11 @@ public:
         /* Todo: can we call VideoModeSupported with the lock held?
          * if not we can introduce a ref counting for the mpView usage
          * to ensure it stays alive till we need it*/
-        HRESULT hr = T::Lock();
+        T::lock();
         HRESULT retHr = S_OK;
-        Assert(hr == S_OK);
-        if (SUCCEEDED(hr))
-        {
-            if (mpView)
-                retHr = T::VideoModeSupported(uWidth, uHeight, uBPP, pbSupported);
-            hr = T::Unlock();
-            Assert(hr == S_OK);
-        }
+        if (mpView)
+            retHr = T::VideoModeSupported(uWidth, uHeight, uBPP, pbSupported);
+        T::unlock();
         return retHr;
     }
 
@@ -2058,17 +1935,13 @@ public:
     void setView(V * pView)
     {
         /* lock to ensure we do not collide with the EMT thread passing commands to us */
-        HRESULT hr = T::Lock();
-        Assert(hr == S_OK);
-        if (SUCCEEDED(hr))
-        {
-            T::setView(pView);
-            mpView = pView;
-            mOverlay.updateAttachment(pView ? pView->viewport() : NULL, pView);
-            hr = T::Unlock();
-            Assert(hr == S_OK);
-        }
+        T::lock();
+        T::setView(pView);
+        mpView = pView;
+        mOverlay.updateAttachment(pView ? pView->viewport() : NULL, pView);
+        T::unlock();
     }
+
 private:
     VBoxQGLOverlay mOverlay;
     V *mpView;
