@@ -79,6 +79,10 @@ public:
     QString versionString() const { return mVerString; }
     bool isBeta() const;
 
+#ifdef Q_WS_MAC
+    static MacOSXRelease osRelease();
+#endif /* Q_WS_MAC */
+
     CVirtualBox virtualBox() const { return mVBox; }
     CHost host() const { return mHost; }
 
@@ -406,10 +410,21 @@ public:
                                     bool fIncludingMachineGeneralCheck = false);
     static bool shouldWeAutoMountGuestScreens(CMachine &machine, bool fIncludingSanityCheck = true);
     static bool shouldWeAllowSnapshotOperations(CMachine &machine, bool fIncludingSanityCheck = true);
+    static MachineCloseAction defaultMachineCloseAction(CMachine &machine);
     static RuntimeMenuType restrictedRuntimeMenuTypes(CMachine &machine);
+#ifdef Q_WS_MAC
+    static RuntimeMenuApplicationActionType restrictedRuntimeMenuApplicationActionTypes(CMachine &machine);
+#endif /* Q_WS_MAC */
+    static RuntimeMenuMachineActionType restrictedRuntimeMenuMachineActionTypes(CMachine &machine);
+    static RuntimeMenuViewActionType restrictedRuntimeMenuViewActionTypes(CMachine &machine);
+    static RuntimeMenuDevicesActionType restrictedRuntimeMenuDevicesActionTypes(CMachine &machine);
+#ifdef VBOX_WITH_DEBUGGER_GUI
+    static RuntimeMenuDebuggerActionType restrictedRuntimeMenuDebuggerActionTypes(CMachine &machine);
+#endif /* VBOX_WITH_DEBUGGER_GUI */
+    static RuntimeMenuHelpActionType restrictedRuntimeMenuHelpActionTypes(CMachine &machine);
     static UIVisualStateType restrictedVisualStateTypes(CMachine &machine);
     static QList<IndicatorType> restrictedStatusBarIndicators(CMachine &machine);
-    static QList<MachineCloseAction> restrictedMachineCloseActions(CMachine &machine);
+    static MachineCloseAction restrictedMachineCloseActions(CMachine &machine);
     static QList<GlobalSettingsPageType> restrictedGlobalSettingsPages(CVirtualBox &vbox);
     static QList<MachineSettingsPageType> restrictedMachineSettingsPages(CMachine &machine);
 
