@@ -44,6 +44,7 @@
 
 #ifdef Q_WS_X11
 # include <X11/Xlib.h>
+# include <dlfcn.h>
 #endif
 
 #include <iprt/buildconfig.h>
@@ -297,6 +298,21 @@ static void showHelp()
     /** @todo Show this as a dialog on windows. */
 }
 
+#ifdef Q_WS_X11
+/** This is a workaround for a bug on old libX11 versions, fixed in commit
+ * 	941f02ede63baa46f93ed8abccebe76fb29c0789 and released in version 1.1. */
+Status VBoxXInitThreads(void)
+{
+    void *pvProcess = dlopen(NULL, RTLD_GLOBAL | RTLD_LAZY);
+    Status rc = 1;
+    if (pvProcess && dlsym(pvProcess, "xcb_connect"))
+        rc = XInitThreads();
+    if (pvProcess)
+        dlclose(pvProcess);
+    return rc;
+}
+#endif
+
 extern "C" DECLEXPORT(int) TrustedMain (int argc, char **argv, char ** /*envp*/)
 {
     LogFlowFuncEnter();
@@ -305,7 +321,7 @@ extern "C" DECLEXPORT(int) TrustedMain (int argc, char **argv, char ** /*envp*/)
 # endif
 
 #ifdef Q_WS_X11
-    if (!XInitThreads())
+    if (!VBoxXInitThreads())
         return 1;
 #endif
 
@@ -330,11 +346,16 @@ extern "C" DECLEXPORT(int) TrustedMain (int argc, char **argv, char ** /*envp*/)
     sigaction (SIGUSR1, &sa, NULL);
 #endif
 
-#ifdef QT_MAC_USE_COCOA
+#ifdef Q_WS_MAC
+     /* Mavericks font fix: */
+    if (VBoxGlobal::osRelease() == MacOSXRelease_Mavericks)
+        QFont::insertSubstitution(".Lucida Grande UI", "Lucida Grande");
+# ifdef QT_MAC_USE_COCOA
     /* Instantiate our NSApplication derivative before QApplication
      * forces NSApplication to be instantiated. */
     UICocoaApplication::instance();
-#endif
+# endif /* QT_MAC_USE_COCOA */
+#endif /* Q_WS_MAC */
 
     qInstallMsgHandler (QtMessageOutput);
 
@@ -560,7 +581,7 @@ int main (int argc, char **argv, char **envp)
      * the selector window. */
     bool fInitSUPLib = false;
 #ifdef Q_WS_X11
-    if (!XInitThreads())
+    if (!VBoxXInitThreads())
         return 1;
 #endif
     for (int i = 1; i < argc; i++)
