@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright (C) 2009-2010 Oracle Corporation
+ * Copyright (C) 2009-2012 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -41,6 +41,9 @@ QString UIMedium::mRow = QString ("<tr><td>%1</td></tr>");
 
 UIMedium& UIMedium::operator= (const UIMedium &aOther)
 {
+    m_fHide = aOther.m_fHide;
+    m_fAttachedToHiddenMachinesOnly = aOther.m_fAttachedToHiddenMachinesOnly;
+
     mMedium = aOther.medium();
     mType = aOther.type();
     mState = aOther.state();
@@ -120,6 +123,10 @@ void UIMedium::blockAndQueryState()
  */
 void UIMedium::refresh()
 {
+    /* We assume these flags are 'false' by default: */
+    m_fHide = false;
+    m_fAttachedToHiddenMachinesOnly = false;
+
     /* Detect basic parameters */
     mId = mMedium.isNull() ? QUuid().toString().remove ('{').remove ('}') : mMedium.GetId();
 
@@ -137,6 +144,16 @@ void UIMedium::refresh()
     mLocation = mMedium.isNull() || mIsHostDrive ? QString ("--") :
                 QDir::toNativeSeparators (mMedium.GetLocation());
 
+    QString tmp;
+    if (!mMedium.isNull())
+        tmp = mMedium.GetProperty("Special/GUI/Hints");
+    if (!tmp.isEmpty())
+    {
+        QStringList tmpList(tmp.split(','));
+        if (tmpList.contains("Hide", Qt::CaseInsensitive))
+            m_fHide = true;
+    }
+
     if (mType == UIMediumType_HardDisk)
     {
         mHardDiskFormat = mMedium.GetFormat();
@@ -194,6 +211,9 @@ void UIMedium::refresh()
         QVector <QString> machineIds = mMedium.GetMachineIds();
         if (machineIds.size() > 0)
         {
+            /* We assume this flag is 'true' if at least one machine present: */
+            m_fAttachedToHiddenMachinesOnly = true;
+
             QString sUsage;
 
             CVirtualBox vbox = vboxGlobal().virtualBox();
@@ -207,7 +227,16 @@ void UIMedium::refresh()
                  * We can skip such a machines in usage string.
                  * CVirtualBox::FindMachine() will return null machine for such case. */
                 if (machine.isNull())
+                {
+                    /* We can't decide for that medium yet,
+                     * assume this flag is 'false' for now: */
+                    m_fAttachedToHiddenMachinesOnly = false;
                     continue;
+                }
+
+                /* Finally, we are checking if current machine overrides this flag: */
+                if (m_fAttachedToHiddenMachinesOnly && vboxGlobal().shouldWeShowMachine(machine))
+                    m_fAttachedToHiddenMachinesOnly = false;
 
                 QString sName = machine.GetName();
                 QString sSnapshots;
