@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright (C) 2006-2011 Oracle Corporation
+ * Copyright (C) 2006-2012 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -771,6 +771,11 @@ void UIMediumManager::mediumAdded (const UIMedium &aMedium)
         (aMedium.isHostDrive()))
         return;
 
+    /* Ignore mediums (and their children) which are marked as hidden or
+     * which are attached to hidden machines only: */
+    if (isMediumAttachedToHiddenMachinesOnly(aMedium))
+        return;
+
     if (!mShowDiffs && aMedium.type() == UIMediumType_HardDisk)
     {
         if (aMedium.parent() && !mSessionMachineId.isNull())
@@ -882,6 +887,11 @@ void UIMediumManager::mediumUpdated (const UIMedium &aMedium)
         (aMedium.isHostDrive()))
         return;
 
+    /* Ignore mediums (and their children) which are marked as hidden or
+     * which are attached to hidden machines only: */
+    if (isMediumAttachedToHiddenMachinesOnly(aMedium))
+        return;
+
     MediaItem *item = 0;
 
     switch (aMedium.type())
@@ -1961,5 +1971,23 @@ QString UIMediumManager::formatPaneText (const QString &aText, bool aCompact /*
     return info;
 }
 
+/* static */
+bool UIMediumManager::isMediumAttachedToHiddenMachinesOnly(const UIMedium &medium)
+{
+    /* Iterate till the root: */
+    const UIMedium *pMedium = &medium;
+    do
+    {
+        /* Ignore medium if its hidden or attached to hidden machines only: */
+        if (pMedium->isHidden())
+            return true;
+        /* Move iterator to parent: */
+        pMedium = pMedium->parent();
+    }
+    while (pMedium);
+    /* False by default: */
+    return false;
+}
+
 #include "UIMediumManager.moc"
 
