@@ -354,6 +354,12 @@ void UIKeyboardHandler::setDebuggerActive(bool aActive /*= true*/)
 #endif /* VBOX_WITH_DEBUGGER_GUI */
 
 #if defined(Q_WS_WIN)
+/** Tell keyboard event handler to skip host keyboard events. Used for HID LEDs sync
+ * when on Windows host a keyboard event is generated in order to change corresponding LED. */
+void UIKeyboardHandler::winSkipKeyboardEvents(bool fSkip)
+{
+    m_fSkipKeyboardEvents = fSkip;
+}
 
 bool UIKeyboardHandler::winEventFilter(MSG *pMsg, ulong uScreenId)
 {
@@ -361,6 +367,11 @@ bool UIKeyboardHandler::winEventFilter(MSG *pMsg, ulong uScreenId)
      * Returning 'true' means filtering-out,
      * Returning 'false' means passing event to Qt. */
     bool fResult = false; /* Pass to Qt by default: */
+
+    /* Skip this event if m_fSkipKeyboardEvents is set by winSkipKeyboardEvents(). */
+    if (m_fSkipKeyboardEvents)
+        return false;
+
     switch (pMsg->message)
     {
         case WM_KEYDOWN:
@@ -672,6 +683,7 @@ UIKeyboardHandler::UIKeyboardHandler(UIMachineLogic *pMachineLogic)
 #if defined(Q_WS_WIN)
     , m_bIsHostkeyInCapture(false)
     , m_iKeyboardHookViewIndex(-1)
+    , m_fSkipKeyboardEvents(false)
 #elif defined(Q_WS_MAC)
     , m_darwinKeyModifiers(0)
     , m_fKeyboardGrabbed(false)
@@ -1017,6 +1029,12 @@ void UIKeyboardHandler::darwinGrabKeyboardEvents(bool fGrab)
         ::CGSetLocalEventsSuppressionInterval(0.0);
         ::darwinSetMouseCoalescingEnabled(false);
 
+        /* Bring the caps lock state up to date, otherwise e.g. a later Shift
+         * key press will accidentally inject a CapsLock key press and release,
+         * see UIKeyboardHandler::darwinKeyboardEvent for the code handling
+         * modifier key state changes */
+        m_darwinKeyModifiers ^= (m_darwinKeyModifiers ^ ::GetCurrentEventKeyModifiers()) & alphaLock;
+
         /* Register the event callback/hook and grab the keyboard. */
         UICocoaApplication::instance()->registerForNativeEvents(RT_BIT_32(10) | RT_BIT_32(11) | RT_BIT_32(12) /* NSKeyDown  | NSKeyUp | | NSFlagsChanged */,
                                                                 UIKeyboardHandler::darwinEventHandlerProc, this);
@@ -1194,11 +1212,11 @@ bool UIKeyboardHandler::keyEventHandleNormal(int iKey, uint8_t uScan, int fFlags
      * we are updating the list of pressed keys and preparing scancodes: */
     if ((fFlags & KeyPressed) || (m_pressedKeys[uScan] & uWhatPressed))
     {
-        /* Check if the guest has the same view on the modifier keys
-         * (NumLock, CapsLock, ScrollLock) as the X server.
-         * If not, send KeyPress events to synchronize the state: */
-        if (fFlags & KeyPressed)
-            fixModifierState(pCodes, puCodesCount);
+        /* If HID LEDs sync is disabled or not supported, check if the guest has the
+         * same view on the modifier keys (NumLock, CapsLock, ScrollLock) as the host. */
+        if (!machineLogic()->isHidLedsSyncEnabled())
+            if (fFlags & KeyPressed)
+                fixModifierState(pCodes, puCodesCount);
 
         /* Prepend 'extended' scancode if needed: */
         if (fFlags & KeyExtended)
