@@ -24,6 +24,7 @@
 #include <QImageWriter>
 #include <QPainter>
 #include <QTimer>
+#include <QDateTime>
 #ifdef Q_WS_MAC
 # include <QMenuBar>
 #endif /* Q_WS_MAC */
@@ -51,6 +52,7 @@
 #include "UISettingsDialogSpecific.h"
 #include "UIVMLogViewer.h"
 #include "UIConverter.h"
+#include "UIExtraDataEventHandler.h"
 #ifdef Q_WS_MAC
 # include "DockIconPreview.h"
 # include "UIExtraDataEventHandler.h"
@@ -76,6 +78,12 @@
 #ifdef VBOX_WITH_DEBUGGER_GUI
 # include <iprt/ldr.h>
 #endif /* VBOX_WITH_DEBUGGER_GUI */
+#ifdef Q_WS_MAC
+# include "DarwinKeyboard.h"
+#endif
+#ifdef Q_WS_WIN
+# include "WinKeyboard.h"
+#endif
 
 /* External includes: */
 #ifdef Q_WS_X11
@@ -192,6 +200,7 @@ void UIMachineLogic::prepare()
     sltMachineStateChanged();
     sltAdditionsStateChanged();
     sltMouseCapabilityChanged();
+    sltSwitchKeyboardLedsToGuestLeds();
 
 #ifdef VBOX_WITH_DEBUGGER_GUI
     /* Prepare debugger: */
@@ -204,6 +213,9 @@ void UIMachineLogic::prepare()
 
 void UIMachineLogic::cleanup()
 {
+    /* Deinitialization: */
+    sltSwitchKeyboardLedsToPreviousLeds();
+
 #ifdef VBOX_WITH_DEBUGGER_GUI
     /* Cleanup debugger: */
     cleanupDebugger();
@@ -260,6 +272,13 @@ UIMachineWindow* UIMachineLogic::activeMachineWindow() const
     return mainMachineWindow();
 }
 
+/** Adjusts guest screen size for each the machine-window we have. */
+void UIMachineLogic::maybeAdjustGuestScreenSize()
+{
+    foreach(UIMachineWindow *pMachineWindow, machineWindows())
+        pMachineWindow->machineView()->maybeAdjustGuestScreenSize();
+}
+
 #ifdef Q_WS_MAC
 void UIMachineLogic::updateDockIcon()
 {
@@ -457,6 +476,35 @@ void UIMachineLogic::sltMouseCapabilityChanged()
         pAction->setChecked(false);
 }
 
+void UIMachineLogic::sltHidLedsSyncStateChanged(bool fEnabled)
+{
+    m_isHidLedsSyncEnabled = fEnabled;
+}
+
+void UIMachineLogic::sltKeyboardLedsChanged()
+{
+    /* Here we have to update host LED lock states using values provided by UISession:
+     * [bool] uisession() -> isNumLock(), isCapsLock(), isScrollLock() can be used for that. */
+
+    if (!isHidLedsSyncEnabled())
+        return;
+
+#if defined(Q_WS_MAC)
+    DarwinHidDevicesBroadcastLeds(m_pHostLedsState, uisession()->isNumLock(), uisession()->isCapsLock(), uisession()->isScrollLock());
+#elif defined(Q_WS_WIN)
+    if (!winHidLedsInSync(uisession()->isNumLock(), uisession()->isCapsLock(), uisession()->isScrollLock()))
+    {
+        keyboardHandler()->winSkipKeyboardEvents(true);
+        WinHidDevicesBroadcastLeds(uisession()->isNumLock(), uisession()->isCapsLock(), uisession()->isScrollLock());
+        keyboardHandler()->winSkipKeyboardEvents(false);
+    }
+    else
+        LogRel2(("HID LEDs Sync: already in sync\n"));
+#else
+    LogRelFlow(("UIMachineLogic::sltKeyboardLedsChanged: Updating host LED lock states does not supported on this platform.\n"));
+#endif
+}
+
 void UIMachineLogic::sltUSBDeviceStateChange(const CUSBDevice &device, bool fIsAttached, const CVirtualBoxErrorInfo &error)
 {
     /* Check if USB device have anything to tell us: */
@@ -500,13 +548,22 @@ void UIMachineLogic::sltGuestMonitorChange(KGuestMonitorChangedEventType, ulong,
         pMachineWindow->handleScreenCountChange();
 }
 
-void UIMachineLogic::sltHostScreenCountChanged(int /*cHostScreenCount*/)
+void UIMachineLogic::sltHostScreenCountChanged()
 {
     /* Deliver event to all machine-windows: */
     foreach (UIMachineWindow *pMachineWindow, machineWindows())
         pMachineWindow->handleScreenCountChange();
 }
 
+void UIMachineLogic::sltHostScreenGeometryChanged()
+{
+    LogRelFlow(("UIMachineLogic: Host-screen geometry changed.\n"));
+
+    /* Deliver event to all machine-windows: */
+    foreach (UIMachineWindow *pMachineWindow, machineWindows())
+        pMachineWindow->handleScreenGeometryChange();
+}
+
 UIMachineLogic::UIMachineLogic(QObject *pParent, UISession *pSession, UIVisualStateType visualStateType)
     : QIWithRetranslateUI3<QObject>(pParent)
     , m_pSession(pSession)
@@ -530,7 +587,29 @@ UIMachineLogic::UIMachineLogic(QObject *pParent, UISession *pSession, UIVisualSt
     , m_pDockPreviewSelectMonitorGroup(0)
     , m_DockIconPreviewMonitor(0)
 #endif /* Q_WS_MAC */
+    , m_pHostLedsState(NULL)
+    , m_isHidLedsSyncEnabled(false)
 {
+    /* Setup HID LEDs synchronization. */
+#if defined(Q_WS_MAC) || defined(Q_WS_WIN)
+    /* Read initial extradata value. */
+    QString strHidLedsSyncSettings = session().GetMachine().GetExtraData(GUI_HidLedsSync);
+
+    /* If extra data GUI/HidLedsSync is not present in VM config or set
+     * to 1 then sync is enabled. Otherwise, it is disabled. */
+
+    /* (temporary disabled by default) */
+    if (strHidLedsSyncSettings == "1")
+        m_isHidLedsSyncEnabled = true;
+    else
+        m_isHidLedsSyncEnabled = false;
+
+    /* Subscribe to GUI_HidLedsSync extradata changes in order to
+     * be able to enable or disable feature dynamically. */
+    connect(gEDataEvents, SIGNAL(sigHidLedsSyncStateChanged(bool)), this, SLOT(sltHidLedsSyncStateChanged(bool)));
+#else
+    m_isHidLedsSyncEnabled = false;
+#endif
 }
 
 void UIMachineLogic::addMachineWindow(UIMachineWindow *pMachineWindow)
@@ -623,6 +702,9 @@ void UIMachineLogic::prepareSessionConnections()
     /* Mouse capability state-change updater: */
     connect(uisession(), SIGNAL(sigMouseCapabilityChange()), this, SLOT(sltMouseCapabilityChanged()));
 
+    /* Keyboard LEDs state-change updater: */
+    connect(uisession(), SIGNAL(sigKeyboardLedsChange()), this, SLOT(sltKeyboardLedsChanged()));
+
     /* USB devices state-change updater: */
     connect(uisession(), SIGNAL(sigUSBDeviceStateChange(const CUSBDevice &, bool, const CVirtualBoxErrorInfo &)),
             this, SLOT(sltUSBDeviceStateChange(const CUSBDevice &, bool, const CVirtualBoxErrorInfo &)));
@@ -640,9 +722,11 @@ void UIMachineLogic::prepareSessionConnections()
     connect(uisession(), SIGNAL(sigGuestMonitorChange(KGuestMonitorChangedEventType, ulong, QRect)),
             this, SLOT(sltGuestMonitorChange(KGuestMonitorChangedEventType, ulong, QRect)));
 
-    /* Host-screen-change updater: */
-    connect(uisession(), SIGNAL(sigHostScreenCountChanged(int)),
-            this, SLOT(sltHostScreenCountChanged(int)));
+    /* Host-screen-change updaters: */
+    connect(uisession(), SIGNAL(sigHostScreenCountChanged()),
+            this, SLOT(sltHostScreenCountChanged()));
+    connect(uisession(), SIGNAL(sigHostScreenGeometryChanged()),
+            this, SLOT(sltHostScreenGeometryChanged()));
 }
 
 void UIMachineLogic::prepareActionGroups()
@@ -940,6 +1024,56 @@ void UIMachineLogic::cleanupActionGroups()
 {
 }
 
+bool UIMachineLogic::eventFilter(QObject *pWatched, QEvent *pEvent)
+{
+    /* Handle machine-window events: */
+    if (UIMachineWindow *pMachineWindow = qobject_cast<UIMachineWindow*>(pWatched))
+    {
+        /* Make sure this window still registered: */
+        if (isMachineWindowsCreated() && m_machineWindowsList.contains(pMachineWindow))
+        {
+            switch (pEvent->type())
+            {
+                /* Handle *window activated* event: */
+                case QEvent::WindowActivate:
+                {
+#ifdef Q_WS_WIN
+                    /* We should save current lock states as *previous* and
+                     * set current lock states to guest values we have,
+                     * As we have no ipc between threads of different VMs
+                     * we are using 100ms timer as lazy sync timout: */
+
+                    /* On Windows host we should do that only in case if sync
+                     * is enabled. Otherwise, keyboardHandler()->winSkipKeyboardEvents(false)
+                     * won't be called in sltSwitchKeyboardLedsToGuestLeds() and guest
+                     * will loose keyboard input forever. */
+                    if (isHidLedsSyncEnabled())
+                    {
+                        keyboardHandler()->winSkipKeyboardEvents(true);
+                        QTimer::singleShot(100, this, SLOT(sltSwitchKeyboardLedsToGuestLeds()));
+                    }
+#else /* Q_WS_WIN */
+                    /* Trigger callback synchronously for now! */
+                    sltSwitchKeyboardLedsToGuestLeds();
+#endif /* !Q_WS_WIN */
+                    break;
+                }
+                /* Handle *window deactivated* event: */
+                case QEvent::WindowDeactivate:
+                {
+                    /* We should restore lock states to *previous* known: */
+                    sltSwitchKeyboardLedsToPreviousLeds();
+                    break;
+                }
+                /* Default: */
+                default: break;
+            }
+        }
+    }
+    /* Call to base-class: */
+    return QIWithRetranslateUI3<QObject>::eventFilter(pWatched, pEvent);
+}
+
 void UIMachineLogic::sltCheckRequestedModes()
 {
     /* Do not try to enter extended mode if machine was not started yet: */
@@ -992,8 +1126,8 @@ void UIMachineLogic::sltAdjustWindow()
         if (pMachineWindow->isMaximized())
             pMachineWindow->showNormal();
 
-        /* Normalize view's geometry: */
-        pMachineWindow->machineView()->normalizeGeometry(true);
+        /* Normalize window geometry: */
+        pMachineWindow->normalizeGeometry(true /* adjust position */);
     }
 }
 
@@ -1993,6 +2127,65 @@ void UIMachineLogic::sltChangeDockIconUpdate(bool fEnabled)
 }
 #endif /* Q_WS_MAC */
 
+void UIMachineLogic::sltSwitchKeyboardLedsToGuestLeds()
+{
+//    /* Log statement (printf): */
+//    QString strDt = QDateTime::currentDateTime().toString("HH:mm:ss:zzz");
+//    printf("%s: UIMachineLogic: sltSwitchKeyboardLedsToGuestLeds called, machine name is {%s}\n",
+//           strDt.toAscii().constData(),
+//           session().GetMachine().GetName().toAscii().constData());
+
+    /* Here we have to save current host LED lock states in UISession registry.
+     * [void] uisession() -> setHostNumLock(), setHostCapsLock(), setHostScrollLock() can be used for that. */
+
+    /* Here we have to update host LED lock states using values provided by UISession registry.
+     * [bool] uisession() -> isNumLock(), isCapsLock(), isScrollLock() can be used for that. */
+
+    if (!isHidLedsSyncEnabled())
+        return;
+
+#if defined(Q_WS_MAC)
+    if (m_pHostLedsState == NULL)
+        m_pHostLedsState = DarwinHidDevicesKeepLedsState();
+    DarwinHidDevicesBroadcastLeds(m_pHostLedsState, uisession()->isNumLock(), uisession()->isCapsLock(), uisession()->isScrollLock());
+#elif defined(Q_WS_WIN)
+    if (m_pHostLedsState == NULL)
+        m_pHostLedsState = WinHidDevicesKeepLedsState();
+    keyboardHandler()->winSkipKeyboardEvents(true);
+    WinHidDevicesBroadcastLeds(uisession()->isNumLock(), uisession()->isCapsLock(), uisession()->isScrollLock());
+    keyboardHandler()->winSkipKeyboardEvents(false);
+#else
+    LogRelFlow(("UIMachineLogic::sltSwitchKeyboardLedsToGuestLeds: keep host LED lock states and broadcast guest's ones does not supported on this platform.\n"));
+#endif
+}
+
+void UIMachineLogic::sltSwitchKeyboardLedsToPreviousLeds()
+{
+//    /* Log statement (printf): */
+//    QString strDt = QDateTime::currentDateTime().toString("HH:mm:ss:zzz");
+//    printf("%s: UIMachineLogic: sltSwitchKeyboardLedsToPreviousLeds called, machine name is {%s}\n",
+//           strDt.toAscii().constData(),
+//           session().GetMachine().GetName().toAscii().constData());
+
+    if (!isHidLedsSyncEnabled())
+        return;
+
+    /* Here we have to restore host LED lock states. */
+    if (m_pHostLedsState)
+    {
+#if defined(Q_WS_MAC)
+    	DarwinHidDevicesApplyAndReleaseLedsState(m_pHostLedsState);
+#elif defined(Q_WS_WIN)
+        keyboardHandler()->winSkipKeyboardEvents(true);
+        WinHidDevicesApplyAndReleaseLedsState(m_pHostLedsState);
+        keyboardHandler()->winSkipKeyboardEvents(false);
+#else
+        LogRelFlow(("UIMachineLogic::sltSwitchKeyboardLedsToPreviousLeds: restore host LED lock states does not supported on this platform.\n"));
+#endif
+        m_pHostLedsState = NULL;
+	}
+}
+
 int UIMachineLogic::searchMaxSnapshotIndex(const CMachine &machine,
                                            const CSnapshot &snapshot,
                                            const QString &strNameTemplate)
