@@ -79,6 +79,9 @@ public:
     bool isPreventAutoClose() const { return m_fIsPreventAutoClose; }
     void setPreventAutoClose(bool fIsPreventAutoClose) { m_fIsPreventAutoClose = fIsPreventAutoClose; }
 
+    /* API: Guest screen size stuff: */
+    virtual void maybeAdjustGuestScreenSize();
+
     /* Wrapper to open Machine settings / Network page: */
     void openNetworkAdaptersDialog() { sltOpenNetworkAdaptersDialog(); }
 
@@ -93,12 +96,16 @@ public:
     void shutdown();
     void powerOff(bool fDiscardingState);
 
+    /** Performs HID LEDs sync. */
+    bool isHidLedsSyncEnabled() { return m_isHidLedsSyncEnabled; };
+
 protected slots:
 
     /* Console callback handlers: */
     virtual void sltMachineStateChanged();
     virtual void sltAdditionsStateChanged();
     virtual void sltMouseCapabilityChanged();
+    virtual void sltKeyboardLedsChanged();
     virtual void sltUSBDeviceStateChange(const CUSBDevice &device, bool fIsAttached, const CVirtualBoxErrorInfo &error);
     virtual void sltRuntimeError(bool fIsFatal, const QString &strErrorId, const QString &strMessage);
 #ifdef RT_OS_DARWIN
@@ -107,7 +114,8 @@ protected slots:
     virtual void sltGuestMonitorChange(KGuestMonitorChangedEventType changeType, ulong uScreenId, QRect screenGeo);
 
     /* Qt callback handler: */
-    virtual void sltHostScreenCountChanged(int cHostScreenCount);
+    virtual void sltHostScreenCountChanged();
+    virtual void sltHostScreenGeometryChanged();
 
 protected:
 
@@ -161,6 +169,9 @@ protected:
     //virtual void cleanupSessionConnections() {}
     //virtual void cleanupRequiredFeatures() {}
 
+    /* Handler: Event-filter stuff: */
+    bool eventFilter(QObject *pWatched, QEvent *pEvent);
+
 private slots:
 
     /* Mode request watch dog: */
@@ -215,6 +226,11 @@ private slots:
     void sltChangeDockIconUpdate(bool fEnabled);
 #endif /* RT_OS_DARWIN */
 
+    /* Handlers: Keyboard LEDs sync logic: */
+    void sltHidLedsSyncStateChanged(bool fEnabled);
+    void sltSwitchKeyboardLedsToGuestLeds();
+    void sltSwitchKeyboardLedsToPreviousLeds();
+
 private:
 
     /* Helpers: */
@@ -256,6 +272,9 @@ private:
     int m_DockIconPreviewMonitor;
 #endif /* Q_WS_MAC */
 
+    void *m_pHostLedsState;
+    bool m_isHidLedsSyncEnabled;
+
     /* Friend classes: */
     friend class UIMachineWindow;
 };
