@@ -38,6 +38,9 @@
 #ifdef VBOX_WITH_VIDEOHWACCEL
 # include "VBoxFBOverlay.h"
 #endif /* VBOX_WITH_VIDEOHWACCEL */
+#ifdef Q_WS_MAC
+# include "VBoxUtils-darwin.h"
+#endif /* Q_WS_MAC */
 
 #ifdef Q_WS_X11
 # include <QX11Info>
@@ -68,6 +71,43 @@
 #include "CUSBController.h"
 #include "CSnapshot.h"
 
+#ifdef Q_WS_MAC
+/**
+ * Application Services: Core Graphics: Display reconfiguration callback.
+ *
+ * Notifies UISession about @a display configuration change.
+ * Corresponding change described by Core Graphics @a flags.
+ * Uses UISession @a pHandler to process this change.
+ *
+ * @note Last argument (@a pHandler) must always be valid pointer to UISession object.
+ * @note Calls for UISession::sltHandleHostDisplayAboutToChange() slot if display configuration changed.
+ */
+void cgDisplayReconfigurationCallback(CGDirectDisplayID display, CGDisplayChangeSummaryFlags flags, void *pHandler)
+{
+    /* Which flags we are handling? */
+    int iHandledFlags = kCGDisplayAddFlag     /* display added */
+                      | kCGDisplayRemoveFlag  /* display removed */
+                      | kCGDisplaySetModeFlag /* display mode changed */;
+
+    /* Handle 'display-add' case: */
+    if (flags & kCGDisplayAddFlag)
+        LogRelFlow(("UISession::cgDisplayReconfigurationCallback: Display added.\n"));
+    /* Handle 'display-remove' case: */
+    else if (flags & kCGDisplayRemoveFlag)
+        LogRelFlow(("UISession::cgDisplayReconfigurationCallback: Display removed.\n"));
+    /* Handle 'mode-set' case: */
+    else if (flags & kCGDisplaySetModeFlag)
+        LogRelFlow(("UISession::cgDisplayReconfigurationCallback: Display mode changed.\n"));
+
+    /* Ask handler to process our callback: */
+    if (flags & iHandledFlags)
+        QTimer::singleShot(0, static_cast<UISession*>(pHandler),
+                           SLOT(sltHandleHostDisplayAboutToChange()));
+
+    Q_UNUSED(display);
+}
+#endif /* Q_WS_MAC */
+
 UISession::UISession(UIMachine *pMachine, CSession &sessionReference)
     : QObject(pMachine)
     /* Base variables: */
@@ -75,10 +115,17 @@ UISession::UISession(UIMachine *pMachine, CSession &sessionReference)
     , m_session(sessionReference)
     /* Common variables: */
     , m_pMenuPool(0)
+    , m_machineStatePrevious(KMachineState_Null)
     , m_machineState(session().GetMachine().GetState())
 #ifdef Q_WS_WIN
     , m_alphaCursor(0)
 #endif /* Q_WS_WIN */
+#ifdef Q_WS_MAC
+    , m_pWatchdogDisplayChange(0)
+#endif /* Q_WS_MAC */
+    , m_defaultCloseAction(MachineCloseAction_Invalid)
+    , m_restrictedCloseActions(MachineCloseAction_Invalid)
+    , m_fAllCloseActionsRestricted(false)
     /* Common flags: */
     , m_fIsFirstTimeStarted(false)
     , m_fIsIgnoreRuntimeMediumsChanging(false)
@@ -94,6 +141,9 @@ UISession::UISession(UIMachine *pMachine, CSession &sessionReference)
     , m_fNumLock(false)
     , m_fCapsLock(false)
     , m_fScrollLock(false)
+    , m_fHostNumLock(false)
+    , m_fHostCapsLock(false)
+    , m_fHostScrollLock(false)
     , m_uNumLockAdaptionCnt(2)
     , m_uCapsLockAdaptionCnt(2)
     /* Mouse flags: */
@@ -203,7 +253,11 @@ void UISession::powerUp()
 
     /* Show "Starting/Restoring" progress dialog: */
     if (isSaved())
+    {
         msgCenter().showModalProgressDialog(progress, machine.GetName(), ":/progress_state_restore_90px.png", mainMachineWindow(), true, 0);
+        /* After restoring from 'saved' state, guest screen size should be adjusted: */
+        machineLogic()->maybeAdjustGuestScreenSize();
+    }
     else
         msgCenter().showModalProgressDialog(progress, machine.GetName(), ":/progress_start_90px.png", mainMachineWindow(), true);
 
@@ -282,6 +336,16 @@ void UISession::powerUp()
                  : "disabled"));
 #endif
 
+/* Check if HID LEDs sync is enabled and add a log message about it. */
+#if defined(Q_WS_MAC) || defined(Q_WS_WIN)
+    if(uimachine()->machineLogic()->isHidLedsSyncEnabled())
+        LogRel(("HID LEDs sync is enabled.\n"));
+    else
+        LogRel(("HID LEDs sync is disabled.\n"));
+#else
+    LogRel(("HID LEDs sync is not supported on this platform.\n"));
+#endif
+
 #ifdef VBOX_GUI_WITH_PIDFILE
     vboxGlobal().createPidfile();
 #endif
@@ -690,6 +754,7 @@ void UISession::sltStateChange(KMachineState state)
     if (m_machineState != state)
     {
         /* Store new data: */
+        m_machineStatePrevious = m_machineState;
         m_machineState = state;
 
         /* Update session settings: */
@@ -741,6 +806,93 @@ void UISession::sltGuestMonitorChange(KGuestMonitorChangedEventType changeType,
     emit sigGuestMonitorChange(changeType, uScreenId, screenGeo);
 }
 
+#ifdef RT_OS_DARWIN
+/**
+ * MacOS X: Restarts display-reconfiguration watchdog timer from the beginning.
+ * @note Watchdog is trying to determine display reconfiguration in
+ *       UISession::sltCheckIfHostDisplayChanged() slot every 500ms for 40 tries.
+ */
+void UISession::sltHandleHostDisplayAboutToChange()
+{
+    LogRelFlow(("UISession::sltHandleHostDisplayAboutToChange()\n"));
+
+    if (m_pWatchdogDisplayChange->isActive())
+        m_pWatchdogDisplayChange->stop();
+    m_pWatchdogDisplayChange->setProperty("tryNumber", 1);
+    m_pWatchdogDisplayChange->start();
+}
+
+/**
+ * MacOS X: Determines display reconfiguration.
+ * @note Calls for UISession::sltHandleHostScreenCountChange() if screen count changed.
+ * @note Calls for UISession::sltHandleHostScreenGeometryChange() if screen geometry changed.
+ */
+void UISession::sltCheckIfHostDisplayChanged()
+{
+    LogRelFlow(("UISession::sltCheckIfHostDisplayChanged()\n"));
+
+    /* Acquire desktop wrapper: */
+    QDesktopWidget *pDesktop = QApplication::desktop();
+
+    /* Check if display count changed: */
+    if (pDesktop->screenCount() != m_screens.size())
+    {
+        /* Recache display data: */
+        recacheDisplayData();
+        /* Reset watchdog: */
+        m_pWatchdogDisplayChange->setProperty("tryNumber", 0);
+        /* Notify listeners about screen-count changed: */
+        return sltHandleHostScreenCountChange();
+    }
+    else
+    {
+        /* Check if at least one display geometry changed: */
+        for (int iScreenIndex = 0; iScreenIndex < pDesktop->screenCount(); ++iScreenIndex)
+        {
+            if (pDesktop->screenGeometry(iScreenIndex) != m_screens.at(iScreenIndex))
+            {
+                /* Recache display data: */
+                recacheDisplayData();
+                /* Reset watchdog: */
+                m_pWatchdogDisplayChange->setProperty("tryNumber", 0);
+                /* Notify listeners about screen-geometry changed: */
+                return sltHandleHostScreenGeometryChange();
+            }
+        }
+    }
+
+    /* Check if watchdog expired, restart if not: */
+    int cTryNumber = m_pWatchdogDisplayChange->property("tryNumber").toInt();
+    if (cTryNumber > 0 && cTryNumber < 40)
+    {
+        /* Restart watchdog again: */
+        m_pWatchdogDisplayChange->setProperty("tryNumber", ++cTryNumber);
+        m_pWatchdogDisplayChange->start();
+    }
+    else
+    {
+        /* Reset watchdog: */
+        m_pWatchdogDisplayChange->setProperty("tryNumber", 0);
+    }
+}
+#endif /* RT_OS_DARWIN */
+
+void UISession::sltHandleHostScreenCountChange()
+{
+    LogRelFlow(("UISession: Host-screen count changed.\n"));
+
+    /* Notify current machine-logic: */
+    emit sigHostScreenCountChanged();
+}
+
+void UISession::sltHandleHostScreenGeometryChange()
+{
+    LogRelFlow(("UISession: Host-screen geometry changed.\n"));
+
+    /* Notify current machine-logic: */
+    emit sigHostScreenGeometryChanged();
+}
+
 void UISession::sltAdditionsChange()
 {
     /* Get our guest: */
@@ -824,12 +976,35 @@ void UISession::prepareConsoleEventHandlers()
 
 void UISession::prepareConnections()
 {
+#ifdef Q_WS_MAC
+    /* Install native display reconfiguration callback: */
+    CGDisplayRegisterReconfigurationCallback(cgDisplayReconfigurationCallback, this);
+#else /* !Q_WS_MAC */
+    /* Install Qt display reconfiguration callbacks: */
     connect(QApplication::desktop(), SIGNAL(screenCountChanged(int)),
-            this, SIGNAL(sigHostScreenCountChanged(int)));
+            this, SLOT(sltHandleHostScreenCountChange()));
+    connect(QApplication::desktop(), SIGNAL(resized(int)),
+            this, SLOT(sltHandleHostScreenGeometryChange()));
+    connect(QApplication::desktop(), SIGNAL(workAreaResized(int)),
+            this, SLOT(sltHandleHostScreenGeometryChange()));
+#endif /* !Q_WS_MAC */
 }
 
 void UISession::prepareScreens()
 {
+#ifdef Q_WS_MAC
+    /* Recache display data: */
+    recacheDisplayData();
+    /* Prepare display-change watchdog: */
+    m_pWatchdogDisplayChange = new QTimer(this);
+    {
+        m_pWatchdogDisplayChange->setInterval(500);
+        m_pWatchdogDisplayChange->setSingleShot(true);
+        connect(m_pWatchdogDisplayChange, SIGNAL(timeout()),
+                this, SLOT(sltCheckIfHostDisplayChanged()));
+    }
+#endif /* Q_WS_MAC */
+
     /* Get machine: */
     CMachine machine = m_session.GetMachine();
 
@@ -873,6 +1048,30 @@ void UISession::loadSessionSettings()
 
     /* Load extra-data settings: */
     {
+        /* Runtime menu settings: */
+#ifdef Q_WS_MAC
+        m_allowedActionsMenuApplication = (RuntimeMenuApplicationActionType)
+                                          (vboxGlobal().restrictedRuntimeMenuApplicationActionTypes(machine) ^
+                                           RuntimeMenuApplicationActionType_All);
+#endif /* Q_WS_MAC */
+        m_allowedActionsMenuMachine     = (RuntimeMenuMachineActionType)
+                                          (vboxGlobal().restrictedRuntimeMenuMachineActionTypes(machine) ^
+                                           RuntimeMenuMachineActionType_All);
+        m_allowedActionsMenuView        = (RuntimeMenuViewActionType)
+                                          (vboxGlobal().restrictedRuntimeMenuViewActionTypes(machine) ^
+                                           RuntimeMenuViewActionType_All);
+        m_allowedActionsMenuDevices     = (RuntimeMenuDevicesActionType)
+                                          (vboxGlobal().restrictedRuntimeMenuDevicesActionTypes(machine) ^
+                                           RuntimeMenuDevicesActionType_All);
+#ifdef VBOX_WITH_DEBUGGER_GUI
+        m_allowedActionsMenuDebugger    = (RuntimeMenuDebuggerActionType)
+                                          (vboxGlobal().restrictedRuntimeMenuDebuggerActionTypes(machine) ^
+                                           RuntimeMenuDebuggerActionType_All);
+#endif /* VBOX_WITH_DEBUGGER_GUI */
+        m_allowedActionsMenuHelp        = (RuntimeMenuHelpActionType)
+                                          (vboxGlobal().restrictedRuntimeMenuHelpActionTypes(machine) ^
+                                           RuntimeMenuHelpActionType_All);
+
         /* Temporary: */
         QString strSettings;
 
@@ -895,6 +1094,15 @@ void UISession::loadSessionSettings()
         m_fReconfigurable = VBoxGlobal::shouldWeAllowMachineReconfiguration(machine);
         updateSessionSettings();
 
+        /* What is the default close action and the restricted are? */
+        m_defaultCloseAction = vboxGlobal().defaultMachineCloseAction(machine);
+        m_restrictedCloseActions = vboxGlobal().restrictedMachineCloseActions(machine);
+        m_fAllCloseActionsRestricted =  (m_restrictedCloseActions & MachineCloseAction_SaveState)
+                                     && (m_restrictedCloseActions & MachineCloseAction_Shutdown)
+                                     && (m_restrictedCloseActions & MachineCloseAction_PowerOff);
+                                     // Close VM Dialog hides PowerOff_RestoringSnapshot implicitly if PowerOff is hidden..
+                                     // && (m_restrictedCloseActions & MachineCloseAction_PowerOff_RestoringSnapshot);
+
 #if 0 /* Disabled for now! */
 # ifdef Q_WS_WIN
         /* Disable host screen-saver if requested: */
@@ -961,6 +1169,14 @@ void UISession::cleanupConsoleEventHandlers()
     UIConsoleEventHandler::destroy();
 }
 
+void UISession::cleanupConnections()
+{
+#ifdef Q_WS_MAC
+    /* Remove display reconfiguration callback: */
+    CGDisplayRemoveReconfigurationCallback(cgDisplayReconfigurationCallback, this);
+#endif /* Q_WS_MAC */
+}
+
 void UISession::updateSessionSettings()
 {
     bool fAllowReconfiguration = m_machineState != KMachineState_Stuck && m_fReconfigurable;
@@ -1390,6 +1606,18 @@ void UISession::setFrameBuffer(ulong uScreenId, UIFrameBuffer* pFrameBuffer)
         m_frameBufferVector[(int)uScreenId] = pFrameBuffer;
 }
 
+#ifdef Q_WS_MAC
+/** MacOS X: Recaches display-configuration data. */
+void UISession::recacheDisplayData()
+{
+    /* Recache display data: */
+    m_screens.clear();
+    QDesktopWidget *pDesktop = QApplication::desktop();
+    for (int iScreenIndex = 0; iScreenIndex < pDesktop->screenCount(); ++iScreenIndex)
+        m_screens << pDesktop->screenGeometry(iScreenIndex);
+}
+#endif /* Q_WS_MAC */
+
 #ifdef VBOX_GUI_WITH_KEYS_RESET_HANDLER
 /**
  * Custom signal handler. When switching VTs, we might not get release events
