@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2007 Oracle Corporation
+ * Copyright (C) 2006-2012 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -42,6 +42,7 @@
 #include "../SUPDrvInternal.h"
 #include <VBox/version.h>
 #include <iprt/asm.h>
+#include <iprt/asm-amd64-x86.h>
 #include <iprt/initterm.h>
 #include <iprt/assert.h>
 #include <iprt/spinlock.h>
@@ -49,6 +50,7 @@
 #include <iprt/process.h>
 #include <iprt/alloc.h>
 #include <iprt/power.h>
+#include <iprt/dbg.h>
 #include <VBox/err.h>
 #include <VBox/log.h>
 
@@ -64,6 +66,9 @@
 #include <IOKit/IOUserClient.h>
 #include <IOKit/pwr_mgt/RootDomain.h>
 #include <IOKit/IODeviceTreeSupport.h>
+#include <IOKit/usb/IOUSBHIDDriver.h>
+#include <IOKit/bluetooth/IOBluetoothHIDDriver.h>
+#include <IOKit/bluetooth/IOBluetoothHIDDriverTypes.h>
 
 #ifdef VBOX_WITH_HOST_VMX
 # include <libkern/version.h>
@@ -99,6 +104,8 @@ static int              VBoxDrvDarwinErr2DarwinErr(int rc);
 static IOReturn         VBoxDrvDarwinSleepHandler(void *pvTarget, void *pvRefCon, UInt32 uMessageType, IOService *pProvider, void *pvMessageArgument, vm_size_t argSize);
 RT_C_DECLS_END
 
+static void             vboxdrvDarwinResolveSymbols(void);
+
 
 /*******************************************************************************
 *   Structures and Typedefs                                                    *
@@ -210,8 +217,14 @@ static PSUPDRVSESSION   g_apSessionHashTab[19];
 /** The number of open sessions. */
 static int32_t volatile g_cSessions = 0;
 /** The notifier handle for the sleep callback handler. */
-static IONotifier *g_pSleepNotifier = NULL;
+static IONotifier      *g_pSleepNotifier = NULL;
 
+/** Pointer to vmx_suspend(). */
+static PFNRT            g_pfnVmxSuspend = NULL;
+/** Pointer to vmx_resume(). */
+static PFNRT            g_pfnVmxResume = NULL;
+/** Pointer to vmx_use_count. */
+static int volatile    *g_pVmxUseCount = NULL;
 
 
 /**
@@ -266,6 +279,8 @@ static kern_return_t    VBoxDrvDarwinStart(struct kmod_info *pKModInfo, void *pv
                         if (g_pSleepNotifier == NULL)
                             LogRel(("VBoxDrv: register for sleep/wakeup events failed\n"));
 
+                        /* Find kernel symbols that are kind of optional. */
+                        vboxdrvDarwinResolveSymbols();
                         return KMOD_RETURN_SUCCESS;
                     }
 
@@ -294,6 +309,39 @@ static kern_return_t    VBoxDrvDarwinStart(struct kmod_info *pKModInfo, void *pv
 }
 
 
+/**
+ * Resolves kernel symbols we want (but may do without).
+ */
+static void vboxdrvDarwinResolveSymbols(void)
+{
+    RTDBGKRNLINFO hKrnlInfo;
+    int rc = RTR0DbgKrnlInfoOpen(&hKrnlInfo, 0);
+    if (RT_SUCCESS(rc))
+    {
+        /* The VMX stuff. */
+        int rc1 = RTR0DbgKrnlInfoQuerySymbol(hKrnlInfo, NULL, "vmx_resume", (void **)&g_pfnVmxResume);
+        int rc2 = RTR0DbgKrnlInfoQuerySymbol(hKrnlInfo, NULL, "vmx_suspend", (void **)&g_pfnVmxSuspend);
+        int rc3 = RTR0DbgKrnlInfoQuerySymbol(hKrnlInfo, NULL, "vmx_use_count", (void **)&g_pVmxUseCount);
+        if (RT_SUCCESS(rc1) && RT_SUCCESS(rc2) && RT_SUCCESS(rc3))
+        {
+            LogRel(("VBoxDrv: vmx_resume=%p vmx_suspend=%p vmx_use_count=%p (%d) cr4=%#x\n",
+                    g_pfnVmxResume, g_pfnVmxSuspend, g_pVmxUseCount, *g_pVmxUseCount, ASMGetCR4() ));
+        }
+        else
+        {
+            LogRel(("VBoxDrv: failed to resolve vmx stuff: vmx_resume=%Rrc vmx_suspend=%Rrc vmx_use_count=%Rrc", rc1, rc2, rc3));
+            g_pfnVmxResume  = NULL;
+            g_pfnVmxSuspend = NULL;
+            g_pVmxUseCount  = NULL;
+        }
+
+        RTR0DbgKrnlInfoRelease(hKrnlInfo);
+    }
+    else
+        LogRel(("VBoxDrv: Failed to open kernel symbols, rc=%Rrc\n", rc));
+}
+
+
 /**
  * Stop the kernel module.
  */
@@ -710,16 +758,16 @@ IOReturn VBoxDrvDarwinSleepHandler(void * /* pvTarget */, void *pvRefCon, UInt32
 
 
 /**
- * Enables or disables VT-x using kernel functions.
- *
- * @returns VBox status code. VERR_NOT_SUPPORTED has a special meaning.
- * @param   fEnable     Whether to enable or disable.
+ * @copydoc SUPR0EnableVTx
  */
 int VBOXCALL supdrvOSEnableVTx(bool fEnable)
 {
 #ifdef VBOX_WITH_HOST_VMX
     int rc;
-    if (version_major >= 10 /* 10 = 10.6.x = Snow Leopard */)
+    if (   version_major >= 10 /* 10 = 10.6.x = Snow Leopard */
+        && g_pfnVmxSuspend
+        && g_pfnVmxResume
+        && g_pVmxUseCount)
     {
         if (fEnable)
         {
@@ -735,11 +783,13 @@ int VBOXCALL supdrvOSEnableVTx(bool fEnable)
                 LogRel(("host_vmxon returned %d\n", rc));
                 rc = VERR_UNRESOLVED_ERROR;
             }
+            LogRel(("VBoxDrv: host_vmxon  -> vmx_use_count=%d rc=%Rrc\n", *g_pVmxUseCount, rc));
         }
         else
         {
             host_vmxoff();
             rc = VINF_SUCCESS;
+            LogRel(("VBoxDrv: host_vmxoff -> vmx_use_count=%d\n", *g_pVmxUseCount));
         }
     }
     else
@@ -755,6 +805,54 @@ int VBOXCALL supdrvOSEnableVTx(bool fEnable)
 }
 
 
+/**
+ * @copydoc SUPR0SuspendVTxOnCpu
+ */
+bool VBOXCALL supdrvOSSuspendVTxOnCpu(void)
+{
+#ifdef VBOX_WITH_HOST_VMX
+    /*
+     * Consult the VMX usage counter, don't try suspend if not enabled.
+     *
+     * Note!  The host_vmxon/off code is still race prone since, but this is
+     *        currently the best we can do without always enable VMX when
+     *        loading the driver.
+     */
+    if (   g_pVmxUseCount
+        && *g_pVmxUseCount > 0)
+    {
+        g_pfnVmxSuspend();
+        return true;
+    }
+    return false;
+#else
+    return false;
+#endif
+}
+
+
+/**
+ * @copydoc SUPR0ResumeVTxOnCpu
+ */
+void VBOXCALL   supdrvOSResumeVTxOnCpu(bool fSuspended)
+{
+#ifdef VBOX_WITH_HOST_VMX
+    /*
+     * Don't consult the counter here, the state knows better.
+     * We're executing with interrupts disabled and anyone racing us with
+     * disabling VT-x will be waiting in the rendezvous code.
+     */
+    if (   fSuspended
+        && g_pfnVmxResume)
+        g_pfnVmxResume();
+    else
+        Assert(!fSuspended);
+#else
+    Assert(!fSuspended);
+#endif
+}
+
+
 bool VBOXCALL supdrvOSGetForcedAsyncTscMode(PSUPDRVDEVEXT pDevExt)
 {
     NOREF(pDevExt);
@@ -816,6 +914,73 @@ void VBOXCALL   supdrvOSLdrUnload(PSUPDRVDEVEXT pDevExt, PSUPDRVLDRIMAGE pImage)
 }
 
 
+/**
+ * Resume Bluetooth keyboard.
+ * If there is no Bluetooth keyboard device connected to the system we just ignore this.
+ */
+static void supdrvDarwinResumeBluetoothKbd(void)
+{
+    OSDictionary *pDictionary = IOService::serviceMatching("AppleBluetoothHIDKeyboard");
+    if (pDictionary)
+    {
+        OSIterator     *pIter;
+        IOBluetoothHIDDriver *pDriver;
+
+        pIter = IOService::getMatchingServices(pDictionary);
+        if (pIter)
+        {
+            while ((pDriver = (IOBluetoothHIDDriver *)pIter->getNextObject()))
+                if (pDriver->isKeyboard())
+                    (void)pDriver->hidControl(IOBTHID_CONTROL_EXIT_SUSPEND);
+
+            pIter->release();
+        }
+        pDictionary->release();
+    }
+}
+
+
+/**
+ * Resume built-in keyboard on MacBook Air and Pro hosts.
+ * If there is no built-in keyboard device attached to the system we just ignore this.
+ */
+static void supdrvDarwinResumeBuiltinKbd(void)
+{
+    /*
+     * AppleUSBTCKeyboard KEXT is responsible for built-in keyboard management.
+     * We resume keyboard by accessing to its IOService. */
+    OSDictionary *pDictionary = IOService::serviceMatching("AppleUSBTCKeyboard");
+    if (pDictionary)
+    {
+        OSIterator     *pIter;
+        IOUSBHIDDriver *pDriver;
+
+        pIter = IOService::getMatchingServices(pDictionary);
+        if (pIter)
+        {
+            while ((pDriver = (IOUSBHIDDriver *)pIter->getNextObject()))
+                if (pDriver->IsPortSuspended())
+                    pDriver->SuspendPort(false, 0);
+
+            pIter->release();
+        }
+        pDictionary->release();
+    }
+}
+
+
+/**
+ * Resume suspended keyboard devices (if any).
+ */
+int VBOXCALL    supdrvDarwinResumeSuspendedKbds(void)
+{
+    supdrvDarwinResumeBuiltinKbd();
+    supdrvDarwinResumeBluetoothKbd();
+
+    return 0;
+}
+
+
 /**
  * Converts an IPRT error code to a darwin error code.
  *
