@@ -4,7 +4,7 @@
 # VirtualBox linux installation script
 
 #
-# Copyright (C) 2007-2011 Oracle Corporation
+# Copyright (C) 2007-2012 Oracle Corporation
 #
 # This file is part of VirtualBox Open Source Edition (OSE), as
 # available from http://www.virtualbox.org. This file is free software;
@@ -392,7 +392,7 @@ if [ "$ACTION" = "install" ]; then
     addrunlevel vboxweb-service 25 75 # This may produce useful output
 
     # Create users group
-    groupadd $GROUPNAME 2> /dev/null
+    groupadd -r -f $GROUPNAME 2> /dev/null
 
     # Create symlinks to start binaries
     ln -sf $INSTALLATION_DIR/VBox.sh /usr/bin/VirtualBox
