@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2011 Oracle Corporation
+ * Copyright (C) 2011-2013 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -1120,7 +1120,10 @@ static int qcowFreeImage(PQCOWIMAGE pImage, bool fDelete)
             RTMemFree(pImage->paL1Table);
 
         if (pImage->pszBackingFilename)
+        {
             RTMemFree(pImage->pszBackingFilename);
+            pImage->pszBackingFilename = NULL;
+        }
 
         qcowL2TblCacheDestroy(pImage);
 
@@ -1241,8 +1244,8 @@ static int qcowOpenImage(PQCOWIMAGE pImage, unsigned uOpenFlags)
                 && pImage->offBackingFilename)
             {
                 /* Load backing filename from image. */
-                pImage->pszFilename = (char *)RTMemAllocZ(pImage->cbBackingFilename + 1); /* +1 for \0 terminator. */
-                if (pImage->pszFilename)
+                pImage->pszBackingFilename = (char *)RTMemAllocZ(pImage->cbBackingFilename + 1); /* +1 for \0 terminator. */
+                if (pImage->pszBackingFilename)
                 {
                     rc = vdIfIoIntFileReadSync(pImage->pIfIo, pImage->pStorage,
                                                pImage->offBackingFilename, pImage->pszBackingFilename,
@@ -2194,7 +2197,8 @@ static int qcowSetOpenFlags(void *pBackendData, unsigned uOpenFlags)
     int rc;
 
     /* Image must be opened and the new flags must be valid. */
-    if (!pImage || (uOpenFlags & ~(VD_OPEN_FLAGS_READONLY | VD_OPEN_FLAGS_INFO | VD_OPEN_FLAGS_ASYNC_IO)))
+    if (!pImage || (uOpenFlags & ~(  VD_OPEN_FLAGS_READONLY | VD_OPEN_FLAGS_INFO
+                                   | VD_OPEN_FLAGS_ASYNC_IO | VD_OPEN_FLAGS_SKIP_CONSISTENCY_CHECKS)))
     {
         rc = VERR_INVALID_PARAMETER;
         goto out;
@@ -2441,7 +2445,7 @@ static int qcowGetParentFilename(void *pBackendData, char **ppszParentFilename)
 
     AssertPtr(pImage);
     if (pImage)
-        if (pImage->pszFilename)
+        if (pImage->pszBackingFilename)
             *ppszParentFilename = RTStrDup(pImage->pszBackingFilename);
         else
             rc = VERR_NOT_SUPPORTED;
