@@ -1193,7 +1193,10 @@ static int qedFreeImage(PQEDIMAGE pImage, bool fDelete)
             RTMemFree(pImage->paL1Table);
 
         if (pImage->pszBackingFilename)
+        {
             RTMemFree(pImage->pszBackingFilename);
+            pImage->pszBackingFilename = NULL;
+        }
 
         qedL2TblCacheDestroy(pImage);
 
@@ -1261,8 +1264,8 @@ static int qedOpenImage(PQEDIMAGE pImage, unsigned uOpenFlags)
                     && (Header.u64FeatureFlags & QED_FEATURE_BACKING_FILE))
                 {
                     /* Load backing filename from image. */
-                    pImage->pszFilename = (char *)RTMemAllocZ(Header.u32BackingFilenameSize + 1); /* +1 for \0 terminator. */
-                    if (pImage->pszFilename)
+                    pImage->pszBackingFilename = (char *)RTMemAllocZ(Header.u32BackingFilenameSize + 1); /* +1 for \0 terminator. */
+                    if (pImage->pszBackingFilename)
                     {
                         pImage->cbBackingFilename  = Header.u32BackingFilenameSize;
                         pImage->offBackingFilename = Header.u32OffBackingFilename;
@@ -2456,7 +2459,7 @@ static int qedGetParentFilename(void *pBackendData, char **ppszParentFilename)
 
     AssertPtr(pImage);
     if (pImage)
-        if (pImage->pszFilename)
+        if (pImage->pszBackingFilename)
             *ppszParentFilename = RTStrDup(pImage->pszBackingFilename);
         else
             rc = VERR_NOT_SUPPORTED;
